#' Function to check if a package is installed and, if false, install it
#'
#' @param package string with the name of the package
#'
#' @return direct package installation if missing
#'
#' @example check.package("ggplot2")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
check.package <- function(package){
if(package %in% rownames(installed.packages()) == FALSE) {install.packages(package)}
}


#' Function to read different databases
#'
#' @param data.path absolute path to data base folder
#'
#' @return dataframe of the simulations read
#'
#' @example impactsM3sys <- ReadDataBase(data.path = "/media/davidnortes/Maxtor/0-SimulationData/EMS15may/Outputs/M3")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
ReadDataBase <- function(data.path){
  check.package("gtools")
  require(gtools)
  aux <- NULL
  datafilelist <- mixedsort(dir(path = data.path, pattern = "*.csv", full.names = F)) # creates the list of all the csv files in the directory
  #datafilelist <- mixedsort(datafilelist)
  for (m in seq_along(datafilelist)) {
    temp <- read.csv(file.path(data.path, datafilelist[m]), header = TRUE)
    temp$rep = m
     aux <- rbind.data.frame(aux, temp)
  }
  return(aux)
  rm(aux,temp,datafilelist)
  gc()
}


#' Function to read different databases and calculate the average.
#'
#' @param data.path absolute path to data base folder
#'
#' @return dataframe of the simulations read
#'
#' @example impactsM3sys <- ReadDataBase(data.path = "/media/davidnortes/Maxtor/0-SimulationData/EMS15may/Outputs/M3")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
ReadDataBase.AVG <- function(data.path){
  check.package("gtools")
  require(gtools)
  aux <- NULL
  datafilelist <- mixedsort(dir(path = data.path, pattern = "*.csv", full.names = F)) # creates the list of all the csv files in the directory
  #datafilelist <- mixedsort(datafilelist)
  for (m in seq_along(datafilelist)) {
    temp <- read.csv(file.path(data.path, datafilelist[m]), header = TRUE)
    temp$rep = m
    aux <- rbind.data.frame(aux, temp)
  }
  aux_AVG <- aggregate(aux[, -c(1:3)], by =list(as.factor(aux$flood.ext), as.factor(aux$fseason), as.factor(aux$dist.ct)), FUN = "mean")
  names(aux_AVG)[1:3] <- c("flood.ext", "fseason", "dist.ct")
  return(aux_AVG)
  rm(aux,datafilelist,temp,aux_AVG)
  gc()
}


#' Function to read different databases especially adapted to M1
#'
#' @param data.path absolute path to data base folder
#' @param tactic choose between "outsourcing" or "autonomous"
#' @param hypothesis choose between "H-" or "H+"
#'
#' @return dataframe of the simulations read
#'
#' @example impactsM1sys <- ReadDataBase_specialM1(data.path = "/media/davidnortes/Maxtor/0-SimulationData/EMS10july/Outputs-Outs/Outputs/M1/impacts/sys/H1", tactic = "autonomous", hypothesis = "H-")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
ReadDataBase_specialM1 <- function(data.path, tactic, hypothesis){
  check.package("gtools")
  require(gtools)
  aux0 <- NULL
  datafilelist <- mixedsort(dir(path = data.path, pattern = "*.csv", full.names = F)) # creates the list of all the csv files in the directory
  #datafilelist <- mixedsort(datafilelist)
  for (m in seq_along(datafilelist)) {
    temp <- read.csv(file.path(data.path, datafilelist[m]), header = TRUE)
    temp <- temp[temp$dist.ct %in% 1,]
    if (tactic == "outsourcing"){
      disc.rate=0.05
      periods <- c(1:(ticks.stop/4))
      discount.factors <- (1+disc.rate)^(1-periods)
      xcor = 50
      nbCoops <-data.frame(c(rep(0, xcor-1), rep(1, 100-xcor+1)))
      colnames(nbCoops) <- "nbCoops"
      nbCoops$HLDBW.winter = nbCoops$nbCoops * 2822400
      nbCoops$WCST.winter = nbCoops$nbCoops * (-564480)
      nbCoops$WCLT.winter = 0
      nbCoops$WCLT.H1 = 0
      nbCoops$WCLT.H2 = ((-564480) * 0.3 * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      nbCoops$WCLT.H2[with(nbCoops, nbCoops == 1)] = ((-564480) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      if (hypothesis == "H-"){
        aux <- temp
        soc <- c(0.45, 0.16, 0.29, 0.1)
        outs_p <- 0.8
        cp <- 2312.64
        aux$TDdirect.LT = aux$TDdirect.LT - aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST - aux$VCAH2.ST - aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TD.ST = aux$TD.ST - aux$VCAH2.ST - aux$HLDBW - aux$WCST 
        aux$TD.LT = aux$TD.LT - aux$WCLT 
        aux$TD = aux$TD - aux$VCAH2.ST - aux$HLDBW - aux$WCST  - aux$WCLT 
        
        aux$VCAH2.ST = aux$VCAH1.ST
        aux$VCAH2.ST[aux$fseason %in% "winter"] = aux$VCAH2.ST[aux$fseason %in% "winter"] + (soc[1]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "spring"] = aux$VCAH2.ST[aux$fseason %in% "spring"] + (soc[2]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "summer"] = aux$VCAH2.ST[aux$fseason %in% "summer"] + (soc[3]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "autumn"] = aux$VCAH2.ST[aux$fseason %in% "autumn"] + (soc[4]*outs_p*cp*100)
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "spring"] = 0
        aux$WCST[aux$fseason %in% "summer"] = 0
        aux$WCST[aux$fseason %in% "autumn"] = 0
        aux$WCLT[aux$fseason %in% "spring"] = ((-564480) * 0.225 * discount.factors[2]) + (sum(rep((-564480) * 0.15, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "summer"] = ((-564480) * 0.3 * discount.factors[2]) + (sum(rep((-564480) * 0.06, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "autumn"] = nbCoops$WCLT.H2[11:99]
        aux$WCLT[aux$fseason %in% "winter"] = 0
        
        aux$TDdirect.LT = aux$TDdirect.LT + aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST + aux$VCAH2.ST + aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TD.ST = aux$TD.ST + aux$VCAH2.ST + aux$HLDBW + aux$WCST 
        aux$TD.LT = aux$TD.LT + aux$WCLT 
        aux$TD = aux$TD + aux$VCAH2.ST + aux$HLDBW + aux$WCST  + aux$WCLT 
        temp <- aux
      }
      if (hypothesis == "H+"){
        aux <- temp
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD = aux$TD - aux$HLDBW - aux$WCST
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD = aux$TD + aux$HLDBW + aux$WCST
        temp <- aux
      }
    }
    
    if (tactic == "autonomous"){
      disc.rate=0.05
      periods <- c(1:(ticks.stop/4))
      discount.factors <- (1+disc.rate)^(1-periods)
      xcor = 50
      nbCoops <-data.frame(c(rep(0, xcor-1), rep(1, 100-xcor+1)))
      colnames(nbCoops) <- "nbCoops"
      nbCoops$HLDBW.winter = nbCoops$nbCoops * 2822400
      nbCoops$HLDBW.autumn = nbCoops$nbCoops * 2822400 * (0.7 - 0.2 * 0.5)
      nbCoops$WCST.winter = nbCoops$nbCoops * (-564480)
      nbCoops$WCLT.winter = 0
      nbCoops$WCLT.H1 = 0
      nbCoops$WCLT.H2 = ((-564480) * (0.3 + 0.2 * 0.5) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      nbCoops$WCLT.H2[with(nbCoops, nbCoops == 1)] = ((-564480) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      if (hypothesis == "H-"){
        aux <- temp
        soc <- c(0.45, 0.16, 0.29, 0.1)
        sod <- c(0.365, 0.185, 0.215, 0.5)
        outs_p <- 0.5
        cp <- 2312.64
        aux$TDdirect.LT = aux$TDdirect.LT - aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST - aux$VCAH2.ST - aux$WCST - aux$HLF
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TD.ST = aux$TD.ST - aux$VCAH2.ST - aux$HLDBW - aux$WCST - aux$HLF
        aux$TD.LT = aux$TD.LT - aux$WCLT 
        aux$TD = aux$TD - aux$VCAH2.ST - aux$HLDBW - aux$WCST  - aux$WCLT - aux$HLF
        
        aux$VCAH2.ST = aux$VCAH1.ST
        aux$VCAH2.ST[aux$fseason %in% "winter"] = aux$VCAH2.ST[aux$fseason %in% "winter"] - (soc[1]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "spring"] = aux$VCAH2.ST[aux$fseason %in% "spring"] - (soc[2]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "summer"] = aux$VCAH2.ST[aux$fseason %in% "summer"] - (soc[3]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "autumn"] = aux$VCAH2.ST[aux$fseason %in% "autumn"] - (soc[4]*outs_p*cp*100)
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$HLDBW[aux$fseason %in% "autumn"] = nbCoops$HLDBW.autumn[11:99]
        aux$WCST[aux$fseason %in% "spring"] = 0
        aux$WCST[aux$fseason %in% "summer"] = 0
        aux$WCST[aux$fseason %in% "autumn"] = 0
        aux$WCLT[aux$fseason %in% "spring"] = ((-564480) * (0.225 + 0.2 * 0.185) * discount.factors[2]) + (sum(rep((-564480) * 0.15, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "summer"] = ((-564480) * (0.3 + 0.2 * 0.215) * discount.factors[2]) + (sum(rep((-564480) * 0.06, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "autumn"] = nbCoops$WCLT.H2[11:99]
        aux$WCLT[aux$fseason %in% "winter"] = ((-564480) * 0.2 * 0.365 * discount.factors[2])
        
        aux$HLF[aux$fseason %in% "spring"] = (723818.2 - 641536) + aux$HLF[aux$fseason %in% "spring"]# *1.2* sod[2]# + aux$HLF[aux$fseason %in% "spring"]*(sod[2]* 0.20)
        aux$HLF[aux$fseason %in% "summer"] = (937888.0 - 851200) + aux$HLF[aux$fseason %in% "summer"]# *1.2* sod[3]# + aux$HLF[aux$fseason %in% "summer"]*(sod[3]* 0.20)
        aux$HLF[aux$fseason %in% "autumn"] = (1052800.0 - 851200) + aux$HLF[aux$fseason %in% "autumn"]#*(sod[4]* 0.20)
        aux$HLF[aux$fseason %in% "winter"] = (217248.0) + aux$HLF[aux$fseason %in% "winter"]# *1.2* sod[1]# + aux$HLF[aux$fseason %in% "winter"]*(sod[1]* 0.20)
        
        aux$TDdirect.LT = aux$TDdirect.LT + aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST + aux$VCAH2.ST + aux$WCST + aux$HLF
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TD.ST = aux$TD.ST + aux$VCAH2.ST + aux$HLDBW + aux$WCST + aux$HLF
        aux$TD.LT = aux$TD.LT + aux$WCLT 
        aux$TD = aux$TD + aux$VCAH2.ST + aux$HLDBW + aux$WCST  + aux$WCLT + aux$HLF 
        temp <- aux
      }
      if (hypothesis == "H+"){
        aux <- temp
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD = aux$TD - aux$HLDBW - aux$WCST
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$HLDBW[aux$fseason %in% "autumn"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD = aux$TD + aux$HLDBW + aux$WCST
        temp <- aux
      }
    }
    temp$rep = m
    aux0 <- rbind.data.frame(aux0, temp)
  }
  return(aux0)
  rm(aux, aux0,temp,datafilelist)
  gc()
}


#' Function to read different databases especially adapted to M2
#'
#' @param data.path absolute path to data base folder
#' @param tactic choose between "outsourcing" or "autonomous"
#' @param hypothesis choose between "H-" or "H+"
#'
#' @return dataframe of the simulations read
#'
#' @example impactsM2sys <- ReadDataBase_specialM2(data.path = "/media/davidnortes/Maxtor/0-SimulationData/EMS10july/Outputs-Outs/Outputs/M2/impacts/sys/H1", tactic = "autonomous", hypothesis = "H-")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
ReadDataBase_specialM2 <- function(data.path, tactic, hypothesis){
  check.package("gtools")
  require(gtools)
  aux0 <- NULL
  datafilelist <- mixedsort(dir(path = data.path, pattern = "*.csv", full.names = F)) # creates the list of all the csv files in the directory
  #datafilelist <- mixedsort(datafilelist)
  for (m in seq_along(datafilelist)) {
    temp <- read.csv(file.path(data.path, datafilelist[m]), header = TRUE)
    temp <- temp[temp$dist.ct %in% 1,]
    if (tactic == "outsourcing"){
      disc.rate=0.05
      periods <- c(1:(ticks.stop/4))
      discount.factors <- (1+disc.rate)^(1-periods)
      xcor = 50
      nbCoops <-data.frame(c(rep(0, xcor-1), rep(1, 100-xcor+1)))
      colnames(nbCoops) <- "nbCoops"
      nbCoops$HLDBW.winter = nbCoops$nbCoops * 2822400
      nbCoops$WCST.winter = nbCoops$nbCoops * (-564480)
      nbCoops$WCLT.winter = 0
      nbCoops$WCLT.H1 = 0
      nbCoops$WCLT.H2 = ((-564480) * 0.3 * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      nbCoops$WCLT.H2[with(nbCoops, nbCoops == 1)] = ((-564480) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      if (hypothesis == "H-"){
        aux <- temp
        aux$TDdirect.LT = aux$TDdirect.LT - aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD.LT = aux$TD.LT - aux$WCLT 
        aux$TD = aux$TD - aux$HLDBW - aux$WCST - aux$WCLT 
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "spring"] = 0
        aux$WCST[aux$fseason %in% "summer"] = 0
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$WCLT[aux$fseason %in% "spring"] = ((-564480) * 0.225 * discount.factors[2]) + (sum(rep((-564480) * 0.15, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "summer"] = ((-564480) * 0.3 * discount.factors[2]) + (sum(rep((-564480) * 0.06, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "autumn"] = nbCoops$WCLT.H2[11:99]
        aux$WCLT[aux$fseason %in% "winter"] = 0
        
        aux$TDdirect.LT = aux$TDdirect.LT + aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD.LT = aux$TD.LT + aux$WCLT 
        aux$TD = aux$TD + aux$HLDBW + aux$WCST + aux$WCLT 
        temp <- aux
      }
      if (hypothesis == "H+"){
        aux <- temp
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD = aux$TD - aux$HLDBW - aux$WCST
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD = aux$TD + aux$HLDBW + aux$WCST
        
        temp <- aux
      }
    }
    
    if (tactic == "autonomous"){
      disc.rate=0.05
      periods <- c(1:(ticks.stop/4))
      discount.factors <- (1+disc.rate)^(1-periods)
      xcor = 50
      nbCoops <-data.frame(c(rep(0, xcor-1), rep(1, 100-xcor+1)))
      colnames(nbCoops) <- "nbCoops"
      nbCoops$HLDBW.winter = nbCoops$nbCoops * 2822400
      nbCoops$HLDBW.autumn = nbCoops$nbCoops * 2822400 * (0.7-0.2*0.5)
      nbCoops$WCST.winter = nbCoops$nbCoops * (-564480)
      nbCoops$WCLT.winter = 0
      nbCoops$WCLT.H1 = 0
      nbCoops$WCLT.H2 = ((-564480) * (0.3 + 0.2 * 0.5) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      nbCoops$WCLT.H2[with(nbCoops, nbCoops == 1)] = ((-564480) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      if (hypothesis == "H-"){
        aux <- temp
        aux$TDdirect.LT = aux$TDdirect.LT - aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD.LT = aux$TD.LT - aux$WCLT 
        aux$TD = aux$TD - aux$HLDBW - aux$WCST - aux$WCLT 
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$HLDBW[aux$fseason %in% "autumn"] = nbCoops$HLDBW.autumn[11:99]
        aux$WCST[aux$fseason %in% "spring"] = 0
        aux$WCST[aux$fseason %in% "summer"] = 0
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$WCLT[aux$fseason %in% "spring"] = ((-564480) * (0.225 + 0.2 * 0.185) * discount.factors[2]) + (sum(rep((-564480) * 0.15, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "summer"] = ((-564480) * (0.3 + 0.2 * 0.215) * discount.factors[2]) + (sum(rep((-564480) * 0.06, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "autumn"] = nbCoops$WCLT.H2[11:99]
        aux$WCLT[aux$fseason %in% "winter"] = ((-564480) * 0.2 * 0.365 * discount.factors[2])
        
        aux$TDdirect.LT = aux$TDdirect.LT + aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD.LT = aux$TD.LT + aux$WCLT 
        aux$TD = aux$TD + aux$HLDBW + aux$WCST + aux$WCLT 
        temp <- aux
      }
      if (hypothesis == "H+"){
        aux <- temp
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD = aux$TD - aux$HLDBW - aux$WCST
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$HLDBW[aux$fseason %in% "autumn"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD = aux$TD + aux$HLDBW + aux$WCST
        temp <- aux
      }
    }
    temp$rep = m
    aux0 <- rbind.data.frame(aux0, temp)
  }
  return(aux0)
  rm(aux,aux0,temp,datafilelist)
  gc()
}


#' Function to read different databases especially adapted to M1
#'
#' @param data.path absolute path to data base folder
#' @param tactic choose between "outsourcing" or "autonomous"
#' @param hypothesis choose between "H-" or "H+"
#'
#' @return dataframe of the simulations read
#'
#' @example impactsM1sys <- ReadDataBase_specialM1(data.path = "/media/davidnortes/Maxtor/0-SimulationData/EMS10july/Outputs-Outs/Outputs/M1/impacts/sys/H1", tactic = "autonomous", hypothesis = "H-")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
ReadDataBase_specialM1.autumn <- function(data.path, tactic, hypothesis){
  check.package("gtools")
  require(gtools)
  aux0 <- NULL
  datafilelist <- mixedsort(dir(path = data.path, pattern = "*.csv", full.names = F)) # creates the list of all the csv files in the directory
  #datafilelist <- mixedsort(datafilelist)
  for (m in seq_along(datafilelist)) {
    temp <- read.csv(file.path(data.path, datafilelist[m]), header = TRUE)
    temp <- temp[temp$dist.ct %in% 1,]
    temp$fseason="autumn"
    if (tactic == "outsourcing"){
      disc.rate=0.05
      periods <- c(1:(ticks.stop/4))
      discount.factors <- (1+disc.rate)^(1-periods)
      xcor = 50
      nbCoops <-data.frame(c(rep(0, xcor-1), rep(1, 100-xcor+1)))
      colnames(nbCoops) <- "nbCoops"
      nbCoops$HLDBW.winter = nbCoops$nbCoops * 2822400
      nbCoops$WCST.winter = nbCoops$nbCoops * (-564480)
      nbCoops$WCLT.winter = 0
      nbCoops$WCLT.H1 = 0
      nbCoops$WCLT.H2 = ((-564480) * 0.3 * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      nbCoops$WCLT.H2[with(nbCoops, nbCoops == 1)] = ((-564480) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      if (hypothesis == "H-"){
        aux <- temp
        soc <- c(0.45, 0.16, 0.29, 0.1)
        outs_p <- 0.8
        cp <- 2312.64
        aux$TDdirect.LT = aux$TDdirect.LT - aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST - aux$VCAH2.ST - aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TD.ST = aux$TD.ST - aux$VCAH2.ST - aux$HLDBW - aux$WCST 
        aux$TD.LT = aux$TD.LT - aux$WCLT 
        aux$TD = aux$TD - aux$VCAH2.ST - aux$HLDBW - aux$WCST  - aux$WCLT 
        
        aux$VCAH2.ST = aux$VCAH1.ST
        aux$VCAH2.ST[aux$fseason %in% "winter"] = aux$VCAH2.ST[aux$fseason %in% "winter"] + (soc[1]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "spring"] = aux$VCAH2.ST[aux$fseason %in% "spring"] + (soc[2]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "summer"] = aux$VCAH2.ST[aux$fseason %in% "summer"] + (soc[3]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "autumn"] = aux$VCAH2.ST[aux$fseason %in% "autumn"] + (soc[4]*outs_p*cp*100)
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "spring"] = 0
        aux$WCST[aux$fseason %in% "summer"] = 0
        aux$WCST[aux$fseason %in% "autumn"] = 0
        aux$WCLT[aux$fseason %in% "spring"] = ((-564480) * 0.225 * discount.factors[2]) + (sum(rep((-564480) * 0.15, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "summer"] = ((-564480) * 0.3 * discount.factors[2]) + (sum(rep((-564480) * 0.06, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "autumn"] = nbCoops$WCLT.H2[11:99]
        aux$WCLT[aux$fseason %in% "winter"] = 0
        
        aux$TDdirect.LT = aux$TDdirect.LT + aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST + aux$VCAH2.ST + aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TD.ST = aux$TD.ST + aux$VCAH2.ST + aux$HLDBW + aux$WCST 
        aux$TD.LT = aux$TD.LT + aux$WCLT 
        aux$TD = aux$TD + aux$VCAH2.ST + aux$HLDBW + aux$WCST  + aux$WCLT 
        temp <- aux
      }
      if (hypothesis == "H+"){
        aux <- temp
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD = aux$TD - aux$HLDBW - aux$WCST
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD = aux$TD + aux$HLDBW + aux$WCST
        temp <- aux
      }
    }
    
    if (tactic == "autonomous"){
      disc.rate=0.05
      periods <- c(1:(ticks.stop/4))
      discount.factors <- (1+disc.rate)^(1-periods)
      xcor = 50
      nbCoops <-data.frame(c(rep(0, xcor-1), rep(1, 100-xcor+1)))
      colnames(nbCoops) <- "nbCoops"
      nbCoops$HLDBW.winter = nbCoops$nbCoops * 2822400
      nbCoops$HLDBW.autumn = nbCoops$nbCoops * 2822400 * (0.7 - 0.2 * 0.5)
      nbCoops$WCST.winter = nbCoops$nbCoops * (-564480)
      nbCoops$WCLT.winter = 0
      nbCoops$WCLT.H1 = 0
      nbCoops$WCLT.H2 = ((-564480) * (0.3 + 0.2 * 0.5) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      nbCoops$WCLT.H2[with(nbCoops, nbCoops == 1)] = ((-564480) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      if (hypothesis == "H-"){
        aux <- temp
        soc <- c(0.45, 0.16, 0.29, 0.1)
        sod <- c(0.365, 0.185, 0.215, 0.5)
        outs_p <- 0.5
        cp <- 2312.64
        aux$TDdirect.LT = aux$TDdirect.LT - aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST - aux$VCAH2.ST - aux$WCST - aux$HLF
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TD.ST = aux$TD.ST - aux$VCAH2.ST - aux$HLDBW - aux$WCST - aux$HLF
        aux$TD.LT = aux$TD.LT - aux$WCLT 
        aux$TD = aux$TD - aux$VCAH2.ST - aux$HLDBW - aux$WCST  - aux$WCLT - aux$HLF
        
        aux$VCAH2.ST = aux$VCAH1.ST
        aux$VCAH2.ST[aux$fseason %in% "winter"] = aux$VCAH2.ST[aux$fseason %in% "winter"] - (soc[1]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "spring"] = aux$VCAH2.ST[aux$fseason %in% "spring"] - (soc[2]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "summer"] = aux$VCAH2.ST[aux$fseason %in% "summer"] - (soc[3]*outs_p*cp*100)
        aux$VCAH2.ST[aux$fseason %in% "autumn"] = aux$VCAH2.ST[aux$fseason %in% "autumn"] - (soc[4]*outs_p*cp*100)
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$HLDBW[aux$fseason %in% "autumn"] = nbCoops$HLDBW.autumn[11:99]
        aux$WCST[aux$fseason %in% "spring"] = 0
        aux$WCST[aux$fseason %in% "summer"] = 0
        aux$WCST[aux$fseason %in% "autumn"] = 0
        aux$WCLT[aux$fseason %in% "spring"] = ((-564480) * (0.225 + 0.2 * 0.185) * discount.factors[2]) + (sum(rep((-564480) * 0.15, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "summer"] = ((-564480) * (0.3 + 0.2 * 0.215) * discount.factors[2]) + (sum(rep((-564480) * 0.06, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "autumn"] = nbCoops$WCLT.H2[11:99]
        aux$WCLT[aux$fseason %in% "winter"] = ((-564480) * 0.2 * 0.365 * discount.factors[2])
        
        aux$HLF[aux$fseason %in% "spring"] = (723818.2 - 641536) + aux$HLF[aux$fseason %in% "spring"]# *1.2* sod[2]# + aux$HLF[aux$fseason %in% "spring"]*(sod[2]* 0.20)
        aux$HLF[aux$fseason %in% "summer"] = (937888.0 - 851200) + aux$HLF[aux$fseason %in% "summer"]# *1.2* sod[3]# + aux$HLF[aux$fseason %in% "summer"]*(sod[3]* 0.20)
        aux$HLF[aux$fseason %in% "autumn"] = (1052800.0 - 851200) + aux$HLF[aux$fseason %in% "autumn"]#*(sod[4]* 0.20)
        aux$HLF[aux$fseason %in% "winter"] = (217248.0) + aux$HLF[aux$fseason %in% "winter"]# *1.2* sod[1]# + aux$HLF[aux$fseason %in% "winter"]*(sod[1]* 0.20)
        
        aux$TDdirect.LT = aux$TDdirect.LT + aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST + aux$VCAH2.ST + aux$WCST + aux$HLF
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TD.ST = aux$TD.ST + aux$VCAH2.ST + aux$HLDBW + aux$WCST + aux$HLF
        aux$TD.LT = aux$TD.LT + aux$WCLT 
        aux$TD = aux$TD + aux$VCAH2.ST + aux$HLDBW + aux$WCST  + aux$WCLT + aux$HLF 
        temp <- aux
      }
      if (hypothesis == "H+"){
        aux <- temp
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD = aux$TD - aux$HLDBW - aux$WCST
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$HLDBW[aux$fseason %in% "autumn"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD = aux$TD + aux$HLDBW + aux$WCST
        temp <- aux
      }
    }
    temp$rep = m
    aux0 <- rbind.data.frame(aux0, temp)
  }
  return(aux0)
  rm(aux, aux0,temp,datafilelist)
  gc()
}


#' Function to read different databases especially adapted to M2
#'
#' @param data.path absolute path to data base folder
#' @param tactic choose between "outsourcing" or "autonomous"
#' @param hypothesis choose between "H-" or "H+"
#'
#' @return dataframe of the simulations read
#'
#' @example impactsM2sys <- ReadDataBase_specialM2(data.path = "/media/davidnortes/Maxtor/0-SimulationData/EMS10july/Outputs-Outs/Outputs/M2/impacts/sys/H1", tactic = "autonomous", hypothesis = "H-")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
ReadDataBase_specialM2.autumn <- function(data.path, tactic, hypothesis){
  check.package("gtools")
  require(gtools)
  aux0 <- NULL
  datafilelist <- mixedsort(dir(path = data.path, pattern = "*.csv", full.names = F)) # creates the list of all the csv files in the directory
  #datafilelist <- mixedsort(datafilelist)
  for (m in seq_along(datafilelist)) {
    temp <- read.csv(file.path(data.path, datafilelist[m]), header = TRUE)
    temp <- temp[temp$dist.ct %in% 1,]
    temp$fseason="autumn"
    if (tactic == "outsourcing"){
      disc.rate=0.05
      periods <- c(1:(ticks.stop/4))
      discount.factors <- (1+disc.rate)^(1-periods)
      xcor = 50
      nbCoops <-data.frame(c(rep(0, xcor-1), rep(1, 100-xcor+1)))
      colnames(nbCoops) <- "nbCoops"
      nbCoops$HLDBW.winter = nbCoops$nbCoops * 2822400
      nbCoops$WCST.winter = nbCoops$nbCoops * (-564480)
      nbCoops$WCLT.winter = 0
      nbCoops$WCLT.H1 = 0
      nbCoops$WCLT.H2 = ((-564480) * 0.3 * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      nbCoops$WCLT.H2[with(nbCoops, nbCoops == 1)] = ((-564480) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      if (hypothesis == "H-"){
        aux <- temp
        aux$TDdirect.LT = aux$TDdirect.LT - aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD.LT = aux$TD.LT - aux$WCLT 
        aux$TD = aux$TD - aux$HLDBW - aux$WCST - aux$WCLT 
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "spring"] = 0
        aux$WCST[aux$fseason %in% "summer"] = 0
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$WCLT[aux$fseason %in% "spring"] = ((-564480) * 0.225 * discount.factors[2]) + (sum(rep((-564480) * 0.15, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "summer"] = ((-564480) * 0.3 * discount.factors[2]) + (sum(rep((-564480) * 0.06, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "autumn"] = nbCoops$WCLT.H2[11:99]
        aux$WCLT[aux$fseason %in% "winter"] = 0
        
        aux$TDdirect.LT = aux$TDdirect.LT + aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD.LT = aux$TD.LT + aux$WCLT 
        aux$TD = aux$TD + aux$HLDBW + aux$WCST + aux$WCLT 
        temp <- aux
      }
      if (hypothesis == "H+"){
        aux <- temp
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD = aux$TD - aux$HLDBW - aux$WCST
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD = aux$TD + aux$HLDBW + aux$WCST
        
        temp <- aux
      }
    }
    
    if (tactic == "autonomous"){
      disc.rate=0.05
      periods <- c(1:(ticks.stop/4))
      discount.factors <- (1+disc.rate)^(1-periods)
      xcor = 50
      nbCoops <-data.frame(c(rep(0, xcor-1), rep(1, 100-xcor+1)))
      colnames(nbCoops) <- "nbCoops"
      nbCoops$HLDBW.winter = nbCoops$nbCoops * 2822400
      nbCoops$HLDBW.autumn = nbCoops$nbCoops * 2822400 * (0.7-0.2*0.5)
      nbCoops$WCST.winter = nbCoops$nbCoops * (-564480)
      nbCoops$WCLT.winter = 0
      nbCoops$WCLT.H1 = 0
      nbCoops$WCLT.H2 = ((-564480) * (0.3 + 0.2 * 0.5) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      nbCoops$WCLT.H2[with(nbCoops, nbCoops == 1)] = ((-564480) * discount.factors[2]) + (sum(rep((-564480) * 0.03, 5) * discount.factors[3:7]))
      if (hypothesis == "H-"){
        aux <- temp
        aux$TDdirect.LT = aux$TDdirect.LT - aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD.LT = aux$TD.LT - aux$WCLT 
        aux$TD = aux$TD - aux$HLDBW - aux$WCST - aux$WCLT 
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$HLDBW[aux$fseason %in% "autumn"] = nbCoops$HLDBW.autumn[11:99]
        aux$WCST[aux$fseason %in% "spring"] = 0
        aux$WCST[aux$fseason %in% "summer"] = 0
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$WCLT[aux$fseason %in% "spring"] = ((-564480) * (0.225 + 0.2 * 0.185) * discount.factors[2]) + (sum(rep((-564480) * 0.15, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "summer"] = ((-564480) * (0.3 + 0.2 * 0.215) * discount.factors[2]) + (sum(rep((-564480) * 0.06, 5) * discount.factors[3:7]))
        aux$WCLT[aux$fseason %in% "autumn"] = nbCoops$WCLT.H2[11:99]
        aux$WCLT[aux$fseason %in% "winter"] = ((-564480) * 0.2 * 0.365 * discount.factors[2])
        
        aux$TDdirect.LT = aux$TDdirect.LT + aux$WCLT 
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD.LT = aux$TD.LT + aux$WCLT 
        aux$TD = aux$TD + aux$HLDBW + aux$WCST + aux$WCLT 
        temp <- aux
      }
      if (hypothesis == "H+"){
        aux <- temp
        aux$TDindirect.ST = aux$TDindirect.ST - aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST - aux$WCST 
        aux$TD.ST = aux$TD.ST - aux$HLDBW - aux$WCST
        aux$TD = aux$TD - aux$HLDBW - aux$WCST
        
        aux$HLDBW[aux$fseason %in% "spring"] = nbCoops$HLDBW.winter[11:99]
        aux$HLDBW[aux$fseason %in% "autumn"] = nbCoops$HLDBW.winter[11:99]
        aux$WCST[aux$fseason %in% "autumn"] = 0
        
        aux$TDindirect.ST = aux$TDindirect.ST + aux$HLDBW
        aux$TDdirect.ST = aux$TDdirect.ST + aux$WCST 
        aux$TD.ST = aux$TD.ST + aux$HLDBW + aux$WCST
        aux$TD = aux$TD + aux$HLDBW + aux$WCST
        temp <- aux
      }
    }
    temp$rep = m
    aux0 <- rbind.data.frame(aux0, temp)
  }
  return(aux0)
  rm(aux,aux0,temp,datafilelist)
  gc()
}


#' Function to subset a dataframe by providing a list of names of columns
#'
#' @param data dataframe of variables to process. It needs to contain columns named flood.ext, fseason, dist.ct, hypothesis
#' @param VariablesList array or vector of arrays. List of variables to extract from dataframe
#'
#' @return melted dataframe to represent 
#'
#' @example impactsM3sys <- DBmelting(data.path = "/media/davidnortes/Maxtor/0-SimulationData/EMS15may/Outputs/M3")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
VbleSelection <- function(data, VariablesList){
  col.num <- which(colnames(data) %in% VariablesList)
  temp <- data[, col.num]
  return(temp)
  rm(temp)
  gc()
}


DBmelting <- function(data, ids){
  check.package("reshape")
  require(reshape)
  temp <- melt(data, id.vars=ids)
  return(temp)
}


#' Function to read different databases and calculate the average.
#'
#' @param data dataframe of variables to process. It needs to contain columns named flood.ext, fseason, dist.ct, hypothesis
#' @param Vble string with the nae of the variable which would be used to subset the dataframe
#' @param ids array of strings with the name of the variables preserved as identificators once the dataframe is melted
#'
#' @return melted dataframe to represent 
#'
#' @example impactsM3sys <- SubsetMelt(data = df, Vble = "VCA", ids = c("flood.ext", "fseason", "dist.ct"))
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
SubsetMelt <- function(data, Vble, ids){
  check.package("reshape")
  require(reshape)
  col.num <- which(colnames(data) %in% Vble)
  temp <- data[, col.num]
  col.num <- which(colnames(data) %in% ids)
  temp2 <- data[, col.num]
  temp3 <- cbind.data.frame(temp2, temp)
  #colnames(temp)[ncol(temp)] <- Vble
  #temp$fseason = as.factor(as.character(temp$fseason))
  temp3 <- melt(temp3, id.vars=ids)
  #temp3$flood.ext = as.numeric(as.character(temp3$flood.ext))
  return(temp3)
  rm(temp,temp2,temp3)
  gc()
}


#' Function to save dataframe made of contrast of Shepiro-Wilks over impact variables in LaTeX tables
#'
#' @param x dataframe with the data to convert to a LaTeX table
#' @param outputfile folder path to save file
#' @param name name of the LaTeX table
#'
#' @return .tex file in the address facilitate in outputfile  
#'
#' @example ToLatexTableNormality(TableContrast, "/home/davidnortes/Thesis/5-Models/ModelsNetLogo/wcs-abm/EMS/EMS-Family/tables/test", "normality") 
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
ToLatexTableNormality <- function(x,outputfile, name){
  check.package("xtable")
  require(xtable)
  dist.table = xtable(x, 
                      caption = "Shapiro-Wilk test results",
                      label = "tab:normality", 
                      align = "rlrr",
                      digits = c(0, 0, 3, 4)) 
  names(dist.table) <- c("Configuration", "Shapiro-Wilk statistic", "p-value")
  print(dist.table,
        floating = TRUE, 
        latex.environments = "center",
        tabular.environment = "tabulary",
        width = "\\textwidth",
        booktabs = TRUE,
        include.colnames = TRUE,
        include.rownames = FALSE,
        file = file.path(outputfile, 
                         paste(name, 
                               ".tex",
                               sep = "")))
}


#' Function to save dataframe made of contrast of Kruskal-Wallis over impact variables in LaTeX tables
#'
#' @param x dataframe with the data to convert to a LaTeX table
#' @param outputfile folder path to save file
#' @param name name of the LaTeX table
#' @param caption table's caption on latex doc
#' @param label table's label for LaTeX's internal references
#' @return .tex file in the address facilitate in outputfile  
#'
#' @example ToLatexTableKW(x= TableContrast, outputfile = getwd(), name = "similarity", caption = "Kruskal-Wallis test results", label = "tab:similarity")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
ToLatexTableKW <- function(x,outputfile,name, caption, label){
  check.package("xtable")
  require(xtable)
  dist.table = xtable(x, 
                      caption = caption,
                      label = label, 
                      align = "rlrr",
                      digits = c(0, 0, 3, 4)) 
  names(dist.table) <- c("Indicator", "Kruskal-Wallis statistic", "p-value")
  
  print(dist.table,
        floating = TRUE, 
        latex.environments = "center",
        tabular.environment = "tabulary",
        width = "\\textwidth",
        booktabs = TRUE,
        include.colnames = TRUE,
        include.rownames = FALSE,
        file = file.path(outputfile, 
                         paste(name, 
                               ".tex",
                               sep = "")))
}


#' Function to save dataframe made of contrast of Mann-Whitney over impact variables in LaTeX tables
#'
#' @param x dataframe with the data to convert to a LaTeX table
#' @param outputfile folder path to save file
#' @param name name of the LaTeX table
#' @param caption table's caption on latex doc
#' @param label table's label for LaTeX's internal references
#' @return .tex file in the address facilitate in outputfile  
#'
#' @example ToLatexTableKW(x= TableContrast, outputfile = getwd(), name = "similarity_pairs1-2", caption = "Mann-Whitney test results", label = "tab:similarity_pairs1-2") 
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
ToLatexTableMW <- function(x,outputfile,name, caption, label){
  check.package("xtable")
  require(xtable)
  dist.table = xtable(x, 
                      caption = caption,
                      label = label, 
                      align = "rlrr",
                      digits = c(0, 0, 3, 4)) 
  names(dist.table) <- c("Indicator", "Mann-Whitney statistic", "p-value")
  
  print(dist.table,
        floating = TRUE, 
        latex.environments = "center",
        tabular.environment = "tabulary",
        width = "\\textwidth",
        booktabs = TRUE,
        include.colnames = TRUE,
        include.rownames = FALSE,
        file = file.path(outputfile, 
                         paste(name, 
                               ".tex",
                               sep = "")))
}


#' Function to simplify dataframes by aggregating columns
#'
#' @param data dataframe 
#' @param VariablesList list of strings with the name of the variables to aggregate
#' @param newvble_name name for the new column with the aggregation
#'
#' @description the function takes the columns in VariablesList and sum them into another column with name "newvble_name". Once it is done, the columns in VariablsList are deleted from the dataframe
#'
#' @return simplified data frame
#'
#' @example df2 <- simpBar(df2, c("DWB", "DFB", "SD", "PD.LT"), "material")
#'
#' @author David Nortes Martinez
#' @date 17/08/2017
simpBar <- function(data, VariablesList, newvble_name){
  require(dplyr)
  col.num <- which(colnames(data) %in% VariablesList)
  data[ , (ncol(data) + 1)] = apply(data[, col.num], 1, "sum")
  names(data)[ncol(data)] <- newvble_name
  data[, col.num] <- NULL
  return(data)
  rm(temp, col.num)
  gc()
}


#' Function to plot stacked bars plots
#'
#' @param data dataframe 
#' @param VblesList list of strings with the name of the variables wished in the plot. Include the variable referring to the season
#' @description VblesList should be listed as follows: variable for X axis, variable with ID of the season, rest of the variables. The procedure will calculate the total sum of the variables provided ignoring those first 3 
#' @param season id of the season in the data frame
#' @param color scale color for bars
#' @param color_label name of each color for the legend
#' @param filename name of the file to store the plot. Vectorial format "pdf"
#' @param filename.path path to save file
#'
#' @return ggplot's plot of  stacked bars
#'
#' @example DamageBarsSeason(data = df, VblesList = c("var1", "var2", "var3"), season = "Spring", filename = "testplot1")
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
DamageBarsSeason <- function(data, VblesList, season, color, color_label, filename, filename.path){
  textsize = 34
  linesize = 1
  check.package("dplyr")
  require(dplyr)
  check.package("ggplot2")
  require(ggplot2)
  check.package("cowplot")
  require(cowplot)
  check.package("scales")
  require(scales)
  check.package("reshape2")
  require(reshape2)
  
  temp1 <- data[data$fseason %in% season,]
  temp1 <-VbleSelection(temp1, VblesList)
  temp1$total = apply(temp1[,-c(1:2)],1,"sum")
  temp2 <- melt(temp1, id.vars=VblesList[1:2])
  names(temp2) <- c("X", "fseason","variable","value")
  temp2$X = as.numeric(as.character(temp2$X))
  temp2$variable <- factor(temp2$variable,  levels = c(VblesList,"total"))
  ggplot(temp2[temp2$variable != "total",], aes(x = X)) +
      scale_x_continuous(breaks = seq(0,100,20), 
                         labels = seq(0,100,20), 
                         limits = c(10, +101)) +
      scale_y_continuous(breaks = seq(-2,13, 2), 
                         limits = c(-2.5, 13)) +
      scale_fill_manual(name="", values=color, labels = color_label) +
      scale_color_manual(name = "", values = "black", labels = "Total impact") +
      theme_bw() +
      theme(panel.grid.major.y=element_line(color = "grey75", size = 0.1),
            panel.grid.minor.y=element_blank(),
            panel.grid.minor.x=element_blank(),
            panel.grid.major.x=element_line(color = "grey75", size = 0.1),
            axis.title = element_text(size = textsize),
            axis.text = element_text(size = textsize),
            axis.ticks = element_line(color = "black"),
            axis.ticks.length = unit(0.25, "cm"),
            strip.text = element_text(size = textsize),
            legend.title = element_text(size = textsize),
            legend.text = element_text(size = textsize-2),
            legend.position = "none",
            plot.title = element_blank(),
            panel.border = element_rect(color = "black", size = 1)) +
     labs(y="", 
           x="")  + 
      guides(fill=guide_legend(nrow=6, byrow=F)) +
      geom_bar(aes(y = value, fill = variable), stat = "identity") + 
      geom_hline(yintercept = 0, color = "grey20") + 
      geom_line(data = temp2[temp2$variable %in% "total",], aes(y=value, color = "Total impact"), size = linesize) 
  ggsave(filename = paste(filename,".pdf", sep=""), 
         plot = last_plot(), 
         device = "pdf", 
         width = 16,         
         height = 6,        
         units = "in",         
         dpi = 300, 
         path = filename.path)
  return(last_plot())  
}


#' Function to represent the winery's networks of a chosen size
#'
#' @param nbCoop number of cooperatives in the network
#' @param nbFarmPerCoop number of farmers by cooperative winery
#' @param nbPlotsPerFarm number of plots owned by each farm
#'
#' @return Three different representations of the same network where 1) links are not explicit; 2) links between winery and farms are explicit, but not between farms and plots; and 3) links between all elements ae explicit. Also the legend in a separate graphic
#'
#' @example gNetEMS(1,10,10)
#'
#' @author David Nortes Martinez
#' @date 3/07/2017
gNetEMS <- function(nbCoop, nbFarmPerCoop, nbPlotsPerFarm){
  check.package("network")
  library(network)
  check.package("GGally")
  library(GGally)
  check.package("ggplot2")
  library(ggplot2)
  check.package("RColorBrewer")
  library(RColorBrewer)
  check.package("sna")
  library(sna)
  check.package("scales")
  library(scales)
  
  qual_col_pals = brewer.pal.info[brewer.pal.info$category == 'qual',]
  col_vector = unlist(mapply(brewer.pal, qual_col_pals$maxcolors, rownames(qual_col_pals)))
  nbElementsNetwork = nbCoop + (nbFarmPerCoop*nbPlotsPerFarm)
  net <- NULL
  col=sample(col_vector, nbFarmPerCoop)
  #netcolor = c("#000000",rep(col, each = nbPlotsPerFarm), col)
  netcolor = c("black",rep("white", each = nbPlotsPerFarm*nbFarmPerCoop), rep("grey60", each = nbFarmPerCoop))
  
  #loop to create a matrix of links between elements in the network. It will assign 1 if there is link; 0 otherwise
  #Each collumn correspond to each farm. By rows, first row is the cooperative winery. plots the following ones
  for(i in seq_len(nbFarmPerCoop)){
    columnA = c(1, rep(0, (i-1)*nbPlotsPerFarm),rep(1, nbPlotsPerFarm))
    columnB = c(rep(0, (nbElementsNetwork - length(columnA))))
    column = c(columnA, columnB)
    net <- cbind(net, column)
  }
  
  net <- matrix(data = net, ncol = nbFarmPerCoop, byrow = F)
  net = network(net)        #transformation of net matrix into a network object
  #assign names and attributes to each node. Order of elments in the network object: Winery - plots (1 to n) - farms (1 to n)
  network.vertex.names(net) <- c("Coop. Winery", rep("Plot", nbFarmPerCoop*nbPlotsPerFarm), rep("Farm", nbFarmPerCoop))
  set.vertex.attribute(net, "nature", c("Coop. Winery", rep("Plot", nbFarmPerCoop*nbPlotsPerFarm), rep("Farm", nbFarmPerCoop)))
  set.vertex.attribute(net, "color", netcolor)
  set.edge.attribute(net, "M1", c( rep("white", nbFarmPerCoop * (1 + nbPlotsPerFarm))))
  set.edge.attribute(net, "M2", c( rep("white", nbFarmPerCoop), rep("grey50", nbFarmPerCoop*nbPlotsPerFarm)))
  set.edge.attribute(net, "M3", c( rep("grey50", nbFarmPerCoop * (1 + nbPlotsPerFarm))))
  #Plotting the network. ggnet is compatible with ggplot2 thus same commands can be used to modify certain functions.
  graphNet <- ggnet2(net, 
                     mode = "fruchtermanreingold",
                     label = F,
                     color = "color",
                     #color = "nature", color.palette =  c("Winery" = "black", "Farm" = "grey40", "Plot" = "grey55"),
                     size = "nature", size.palette = c("Coop. Winery" = 19, "Farm" = 7, "Plot" = 3),
                     edge.color = "grey50") +
    geom_point(aes(color = color, size = size), shape = 1,  color = "black") +
    guides(color = FALSE) +
    theme(legend.title = element_blank(),
          legend.position = "none")
  print(graphNet)
  
  cor <- gplot.layout.fruchtermanreingold(net, NULL)
  net %v% "x" = cor[,1] 
  net %v% "y" = cor[,2]
  textsize = 20
  M1 <<- ggnet2(net, 
                mode = c("x", "y"),
                label = F,
                color = "color",
                #color = "nature", color.palette =  c("Winery" = "black", "Farm" = "grey40", "Plot" = "grey55"),
                size = "nature", size.palette = c("Coop. Winery" = 19, "Farm" = 5, "Plot" = 1), 
                edge.color = "M1") +
    geom_point(aes(color = color, size = size), shape = 1,  color = "black") +
    guides(color = FALSE) +
    theme(legend.title = element_blank(),
          legend.position = "none",
          legend.text = element_text(size = textsize))
  print(M1)
  M2 <<- ggnet2(net, 
                mode = c("x", "y"),
                label = F,
                color = "color",
                #color = "nature", color.palette =  c("Winery" = "black", "Farm" = "grey40", "Plot" = "grey55"),
                size = "nature", size.palette = c("Coop. Winery" = 19, "Farm" = 5, "Plot" = 1),
                edge.color = "M2") +
    geom_point(aes(color = color, size = size), shape = 1,  color = "black") +
    guides(color = FALSE) +
    theme(legend.title = element_blank(),
          legend.position = "none",
          legend.text = element_text(size = textsize))
  print(M2)
  M3 <<- ggnet2(net, 
                mode = c("x", "y"),
                label = F,
                color = "color",
                #color = "nature", color.palette =  c("Winery" = "black", "Farm" = "grey40", "Plot" = "grey55"),
                size = "nature", size.palette = c("Coop. Winery" = 19, "Farm" = 5, "Plot" = 1),
                edge.color = "M3")+
    geom_point(aes(color = color, size = size), shape = 1,  color = "black") +
    guides(color = FALSE) +
    theme(legend.title = element_blank(),
          legend.position = "none",
          legend.text = element_text(size = textsize))
  print(M3)
  for(i in seq_len(3)){
    last_plot() + theme(legend.text = element_text(size = 20)) 
    ggsave(filename = paste("NetworkM",i,"size",nbFarmPerCoop,".pdf", sep = ""), 
           plot = get(paste("M",i, sep = "")),
           device = "pdf", 
           width = 8,         
           height = 8,        
           units = "in",         
           dpi = 300, 
           path = "/home/davidnortes/Thesis/2-Documents/ArticlesThesis/ArticleEMS/figure")
  }
  pdf("/home/davidnortes/Thesis/2-Documents/ArticlesThesis/ArticleEMS/figure/Network_legend.pdf",
      width=4, 
      height=0.7)
  par(mai=c(.1,.1,.1,.1))
  plot.new() 
  legend("center",
         col = c(rep("black", 3)),
         pt.bg = c("white","grey60","black"),
         pt.cex=c(1,2,3),
         pch = c(rep(21, 3)),
         legend = c("Plot", "Farm", "Coop. winery"), 
         #legend = c("Winery", "Farm", "Plot"), 
         bty = "n",
         horiz = T)
  dev.off()
}


#' Function to launch netlogo simulations
#'
#' @param nl.id String identifying the Netlogo instance to which the information is sent. Same as used in NLStart and NLLoadModel
#' @param ticks.stop number of ticks allowed in each simulation
#' @param tactic String indicating the farm's tactic to be adopted in case of flood. For the moment limited to "autonomous" and "outsourcing"
#' @param config Integer among c(1,6:14) to identify the which plot belongs to which farm
#' @param season string indicating the season in which the flood will happen: "spring", "summer", "autumn", "winter"
#' @param floodextent Extent of the flood to be simulated. Integer between [0, 100]
#' @param dam_byR Boolean: TRUE to indicate Netlogo to look for destroyed plots in the list passed by "plotdam". FALSE to make netlogo use its internal random number generator to generate the list of destroyed plots
#' @param plotdam Either a list or a vector of plots to be passed as destroyed when the flood takes place. If dam_byR = "FALSE", plotdam should be empty
#'
#' @return list of simulated values 
#'
#' @example launchsim(nl.id = "instance1", ticks.stop = 120, tactic = "outsourcing", config = 1, season = "spring", floodextent = 50, dam_byR = TRUE, plotdam = c(34, 45, 76, 28))
#' @example launchsim(nl.id = "instance2", ticks.stop = 120, tactic = "autonomous", config = 8, season = "autumn", floodextent = 70, dam_byR = FALSE, plotdam = c())
#'
#' @author David Nortes Martinez
#' @date 26/07/2017
launchsim <- function(nl.id, ticks.stop, tactic, config, season, floodextent, dam_byR, plotdam){
  nbseason <- c("spring" = 1,
                "summer" = 2,
                "autumn" = 3,
                "winter" = 4)
  NLCommand(sprintf("set ticks-stop %s", ticks.stop), nl.obj = nl.id)
  NLCommand(sprintf('set TacticalBehavior "%s"', tactic), nl.obj = nl.id)
  NLCommand(sprintf("set dist-control %s", config), nl.obj = nl.id)
  NLCommand(sprintf('set flood-in-season "%s"', season), nl.obj = nl.id)
  NLCommand(sprintf("set Flood-Extent %s", floodextent), nl.obj = nl.id)
  NLCommand(sprintf('set dam_byR %s', dam_byR), nl.obj = nl.id)
  NLCommand(sprintf("set flood-in-season-nb %s", nbseason[[season]]), nl.obj = nl.id)
  NLCommand("set maxExtent 100", nl.obj = nl.id)
  NLCommand('set CatNat-CA? "no"', nl.obj = nl.id)
  NLCommand("set replant? 1", nl.obj = nl.id)
  NLCommand("setup", nl.obj = nl.id)
  NLCommand(sprintf("set cropland-dest [%s]", plotdam), nl.obj = nl.id)
  NLDoCommand(ticks.stop, "go", nl.obj = nl.id)
  sim.results <- NLReport("results", nl.obj = nl.id)
  return(sim.results)
}


#' Function to launch netlogo simulations specifiying the position of the cooperative winery 
#'
#' @param nl.id String identifying the Netlogo instance to which the information is sent. Same as used in NLStart and NLLoadModel
#' @param ticks.stop number of ticks allowed in each simulation
#' @param tactic String indicating the farm's tactic to be adopted in case of flood. For the moment limited to "autonomous" and "outsourcing"
#' @param config Integer among c(1,6:14) to identify the which plot belongs to which farm
#' @param season string indicating the season in which the flood will happen: "spring", "summer", "autumn", "winter"
#' @param floodextent Extent of the flood to be simulated. Integer between [0, 100]
#' @param dam_byR Boolean: TRUE to indicate Netlogo to look for destroyed plots in the list passed by "plotdam". FALSE to make netlogo use its internal random number generator to generate the list of destroyed plots
#' @param plotdam Either a list or a vector of plots to be passed as destroyed when the flood takes place. If dam_byR = "FALSE", plotdam should be empty
#' @param winpos string providing the position of the cooperative winery. Accepted values: "safe" or "prone"
#'
#' @return list of simulated values 
#'
#' @example launchsim_winpos(nl.id = "instance1", ticks.stop = 120, tactic = "outsourcing", config = 1, season = "spring", floodextent = 50, dam_byR = TRUE, plotdam = c(34, 45, 76, 28), winpos = 10)
#' @example launchsim_winpos(nl.id = "instance2", ticks.stop = 120, tactic = "autonomous", config = 8, season = "autumn", floodextent = 70, dam_byR = FALSE, plotdam = c(), winpos = 100)
#'
#' @author David Nortes Martinez
#' @date 26/07/2017
launchsim_winpos <- function(nl.id, ticks.stop, tactic, config, season, floodextent, dam_byR, plotdam, winpos){
  nbseason <- c("spring" = 1,
                "summer" = 2,
                "autumn" = 3,
                "winter" = 4)
  winpos_transl <- c("safe" = 115,
                     "prone" = 10)
  NLCommand(sprintf("set ticks-stop %s", ticks.stop), nl.obj = nl.id)
  NLCommand(sprintf('set TacticalBehavior "%s"', tactic), nl.obj = nl.id)
  NLCommand(sprintf("set dist-control %s", config), nl.obj = nl.id)
  NLCommand(sprintf('set flood-in-season "%s"', season), nl.obj = nl.id)
  NLCommand(sprintf("set Flood-Extent %s", floodextent), nl.obj = nl.id)
  NLCommand(sprintf('set dam_byR %s', dam_byR), nl.obj = nl.id)
  NLCommand(sprintf("set flood-in-season-nb %s", nbseason[[season]]), nl.obj = nl.id)
  NLCommand("set maxExtent 100", nl.obj = nl.id)
  NLCommand('set CatNat-CA? "no"', nl.obj = nl.id)
  NLCommand("set replant? 1", nl.obj = nl.id)
  NLCommand("setup", nl.obj = nl.id)
  NLCommand(sprintf("set cropland-dest [%s]", plotdam), nl.obj = nl.id)
  NLSetAgentSet("cooperatives", winpos_transl[[winpos]], "xcor", nl.obj = nl.id)
  NLDoCommand(ticks.stop, "go", nl.obj = nl.id)
  sim.results <- NLReport("results", nl.obj = nl.id)
  return(sim.results)
}


#' Function to convert strings with the damaged plots into lists of integers
#'
#' @param ch String of plots to be passed as destroyed when the flood takes place.
#' #'
#' @return vector of integers with the id numbers of the amaged plots
#'
#' @example ch_to_nl.list(ch = test$who[[16]])
#'
#' @author Frédéric Grelot
#' @date 28/07/2017
ch_to_nl.list <- function(ch){
  paste(eval(parse(text = ch)), collapse = " ")   
}


#' Function to calculate the components of change for wine-making cost
#'
#' @param df dataframe of nl simulation 
#' #'
#' @return same dataframe with components of wine-making costs changes calculated
#'
#' @example WineMakingComp(df = m3_autumn_30_conf1_autonomous_prone_rep2)
#'
#' @author David Nortes Martinez
#' @date 28/07/2017
WineMakingComp <- function(df){
  aux <- df
  aux$delta.cost = aux$vinification.cost - aux$t.1.vinification.cost
  aux$delta.cost.aggregation = aux$vinif.cost.change.self +  aux$vinif.cost.change.others
  aux$delta.adjustment = aux$delta.cost / aux$delta.cost.aggregation
  aux$delta.adjustment[is.infinite(aux$delta.adjustment)] <- 0
  aux$vinif.cost.change.self.adjusted = aux$delta.adjustment * aux$vinif.cost.change.self
  aux$vinif.cost.change.others.adjusted = aux$delta.adjustment * aux$vinif.cost.change.others
  aux$delta.cost.agg.adjusted = aux$vinif.cost.change.self.adjusted + aux$vinif.cost.change.others.adjusted
  aux[is.na(aux)] <- 0
  return(aux)
}


#' Function to manipulate the netlogo data
#'
#' @param df dataframe of nl simulation 
#' #'
#' @return same dataframe with components of wine-making costs changes calculated
#'
#' @example WineMakingComp(df = m3_autumn_30_conf1_autonomous_prone_rep2)
#'
#' @author David Nortes Martinez
#' @date 28/07/2017
data.manip <- function(df, floodextent, config, season, ticks.stop, disc.rate){
  wineprice = 80
  periods <- c(1:(ticks.stop/4))
  discount.factors <- (1+disc.rate)^(1-periods)
  n <- nrow(df)
  # creation of the new vector of results 'x'
  x <- c(as.integer(df$agent[1]),                                                                    #v1 #identify the farm
         df$revenue[1]*discount.factors[1],                                                          #v3 #value of the revenues form cooperative in the first yer
         sum(df$revenue[2:n] * discount.factors[2:n]),                                               #v4 #value of the revenues from cooperative in years 2 to 6
         df$vinification.cost [1] * discount.factors[1],                                             #v6 #value of the costs form cooperative in the first yer
         sum(df$vinification.cost[2:n] * discount.factors[2:n]),                                     #v8 #value of the cost from cooperative in years 6 to end
         df$vinif.cost.change.self.adjusted[1] * discount.factors[1],                                #value of the costs form cooperative in the first yer
         sum(df$vinif.cost.change.self.adjusted[2:n] * discount.factors[2:n]),                       #value of the cost from cooperative in years 2 to 6
         df$vinif.cost.change.others.adjusted[1] * discount.factors[1],                              #value of the costs form cooperative in the first yer
         sum(df$vinif.cost.change.others.adjusted[2:n] * discount.factors[2:n]),                     #value of the cost from cooperative in years 2 to 6
         (df$reparations[1] - df$soil.reparation[1] - df$plant.reparation[1]) * discount.factors[1], #value of the cost of farm's reparations on buildings in year 1
         sum(df$reparation.cost.from.cooperative[1:n] * discount.factors[1:n]),                      #value of the cost of winery's reparations on buildings in year 1
         df$outsourcing.cost[1] * discount.factors[1],                                               #value of the harvest lost because of damages in farm's buildings in year 1
         sum(df$harvest.lost.bcwinery[1:n] * wineprice * discount.factors[1:n]),                     #value of the harvest lost because of damages in winery's buildings in year 1
         df$soil.reparation[1] * discount.factors[1],                                                #value of the cost of farm's reparations on soils in year 1
         df$plant.reparation[1] * discount.factors[1],                                               #value of the cost of farm's reparations on plants in year 1
         sum(df$plant.repositions[1:n] * discount.factors[1:n]),                                     #value of the cost of farm's reparations on plants in year 1
         (df$harvest.damaged.bcplants[1] * wineprice) * discount.factors[1],                         #value of the cost of farm's reparations on buildings in year 2
         sum(df$total.harvested[2:n] * wineprice * discount.factors[2:n]),                           #value of the cost from cooperative in years 6 to end
         df$harvest.damaged[1] * wineprice * discount.factors[1],                                    #value of the cost of farm's reparations on buildings in year 2
         df$production.cost[1] * discount.factors[1],                                                #45 is the average value of production cost per hl in our model 
         sum(df$production.cost[2:n] * discount.factors[2:n]),                                       #value of the variable cost avoided in farms year 2-5. Unitary value 1
         floodextent,                                                                                #value of the cost from cooperative in years 6 to end
         config,                                                                                     #value of the cost from cooperative in years 6 to end
         season)                                                                                     #value of the cost from cooperative in years 6 to end
  names(x) <- c("Farm", 
                 "RST", 
                 "RLT", 
                 "WCST",
                 "WCLT", 
                 "WCST-CS",
                 "WCLT-CS",
                 "WCST-CO",
                 "WCLT-CO",
                 "DFB",
                 "DWB",
                 "OUTS",
                 "HLDBW",
                 "SD",
                 "PD",
                 "PR",
                 "HLPD.ST",
                 "HLPD.LT",
                 "HLF",
                 "VCA-ST",
                 "VCA-LT",
                 "flood.ext",
                 "dist.ct",
                 "fseason")
  return(x)
}


#' Function to manipulate the netlogo data. M2 branch
#'
#' @param df dataframe of nl simulation 
#' #'
#' @return same dataframe with components of wine-making costs changes calculated
#'
#' @example WineMakingComp(df = m3_autumn_30_conf1_autonomous_prone_rep2)
#'
#' @author David Nortes Martinez
#' @date 28/07/2017
data.manip.m2 <- function(df, floodextent, config, season, ticks.stop, disc.rate){
  wineprice = 80
  periods <- c(1:(ticks.stop/4))
  discount.factors <- (1+disc.rate)^(1-periods)
  n <- nrow(df)
  # creation of the new vector of results 'x'
  x <- c(as.integer(df$agent[1]),                                                                    #v1 #identify the farm
         df$revenue[1]*discount.factors[1],                                                          #v3 #value of the revenues form cooperative in the first yer
         sum(df$revenue[2:n] * discount.factors[2:n]),                                               #v4 #value of the revenues from cooperative in years 2 to 6
         (df$reparations[1] - df$soil.reparation[1] - df$plant.reparation[1]) * discount.factors[1], #value of the cost of farm's reparations on buildings in year 1
         df$outsourcing.cost[1] * discount.factors[1],                                               #value of the harvest lost because of damages in farm's buildings in year 1
         df$soil.reparation[1] * discount.factors[1],                                                #value of the cost of farm's reparations on soils in year 1
         df$plant.reparation[1] * discount.factors[1],                                               #value of the cost of farm's reparations on plants in year 1
         sum(df$plant.repositions[1:n] * discount.factors[1:n]),                                     #value of the cost of farm's reparations on plants in year 1
         (df$harvest.damaged.bcplants[1] * wineprice) * discount.factors[1],                         #value of the cost of farm's reparations on buildings in year 2
         sum(df$total.harvested[2:n] * wineprice * discount.factors[2:n]),                           #value of the cost from cooperative in years 6 to end
         df$harvest.damaged[1] * wineprice * discount.factors[1],                                    #value of the cost of farm's reparations on buildings in year 2
         df$production.cost[1] * discount.factors[1],                                                #45 is the average value of production cost per hl in our model 
         sum(df$production.cost[2:n] * discount.factors[2:n]),                                       #value of the variable cost avoided in farms year 2-5. Unitary value 1
         floodextent,                                                                                #value of the cost from cooperative in years 6 to end
         config,                                                                                     #value of the cost from cooperative in years 6 to end
         season)                                                                                     #value of the cost from cooperative in years 6 to end
  names(x) <- c("Farm", 
                "RST", 
                "RLT", 
                "DFB",
                "OUTS",
                "SD",
                "PD",
                "PR",
                "HLPD.ST",
                "HLPD.LT",
                "HLF",
                "VCA-ST",
                "VCA-LT",
                "flood.ext",
                "dist.ct",
                "fseason")
  return(x)
}


#' Function to manipulate the netlogo data. M1 branch
#'
#' @param df dataframe of nl simulation 
#' #'
#' @return same dataframe with components of wine-making costs changes calculated
#'
#' @example WineMakingComp(df = m3_autumn_30_conf1_autonomous_prone_rep2)
#'
#' @author David Nortes Martinez
#' @date 28/07/2017
data.manip.m1 <- function(df, floodextent, config, season, ticks.stop, disc.rate){
  wineprice = 80
  periods <- c(1:(ticks.stop/4))
  discount.factors <- (1+disc.rate)^(1-periods)
  n <- nrow(df)
  # creation of the new vector of results 'x'
  x <- c(as.integer(df$agent[1]),                                                                    #v1 #identify the farm
         df$revenue[1]*discount.factors[1],                                                          #v3 #value of the revenues form cooperative in the first yer
         sum(df$revenue[2:n] * discount.factors[2:n]),                                               #v4 #value of the revenues from cooperative in years 2 to 6
         (df$reparations[1] - df$soil.reparation[1] - df$plant.reparation[1]) * discount.factors[1], #value of the cost of farm's reparations on buildings in year 1
         df$soil.reparation[1] * discount.factors[1],                                                #value of the cost of farm's reparations on soils in year 1
         df$plant.reparation[1] * discount.factors[1],                                               #value of the cost of farm's reparations on plants in year 1
         sum(df$plant.repositions[1:n] * discount.factors[1:n]),                                     #value of the cost of farm's reparations on plants in year 1
         (df$harvest.damaged.bcplants[1] * wineprice) * discount.factors[1],                         #value of the cost of farm's reparations on buildings in year 2
         sum(df$total.harvested[2:n] * wineprice * discount.factors[2:n]),                           #value of the cost from cooperative in years 6 to end
         df$harvest.damaged[1] * wineprice * discount.factors[1],                                    #value of the cost of farm's reparations on buildings in year 2
         df$production.costH1[1] * discount.factors[1],                                                #45 is the average value of production cost per hl in our model 
         sum(df$production.costH1[2:n] * discount.factors[2:n]),                                       #value of the variable cost avoided in farms year 2-5. Unitary value 1
         df$production.costH2[1] * discount.factors[1],                                                #45 is the average value of production cost per hl in our model 
         sum(df$production.costH2[2:n] * discount.factors[2:n]),                                       #value of the variable cost avoided in farms year 2-5. Unitary v
         floodextent,                                                                                #value of the cost from cooperative in years 6 to end
         config,                                                                                     #value of the cost from cooperative in years 6 to end
         season)                                                                                     #value of the cost from cooperative in years 6 to end
  names(x) <- c("Farm", 
                "RST", 
                "RLT", 
                "DFB",
                "SD",
                "PD",
                "PR",
                "HLPD.ST",
                "HLPD.LT",
                "HLF",
                "VCAH1-ST",
                "VCAH1-LT",
                "VCAH2-ST",
                "VCAH2-LT",
                "flood.ext",
                "dist.ct",
                "fseason")
  return(x)
}



#' Function to calculate the components of change for wine-making cost
#'
#' @param array array containing both the winery position and the tactic used by farms when flooded, codified as in the Experiment plan database 
#' #'
#' @return list with the winery position and the tactic used
#'
#' @example wintac_finder(datafilelist[1]) where  datafilelist[1] is "m3_autumn_flood11_conf1_outsourcing_safe_rep1.csv"
#'
#' @author David Nortes Martinez
#' @date 31/07/2017
wintac_finder <- function(array){
  coop <- ifelse(grepl("safe", array), "safe", "prone")
  strategy <- ifelse(grepl("autonomous", array), "autonomous", "outsourcing")
  
  return(list('winpos' = coop, 'tactic' = strategy))
  ## returns the first location of 'b', in this case: 2
}

#' Function to calculate the impacts of a flood
#'
#' @param data.path_fl path to the data folder where the results of the fllod simulations are stored
#' @param data.path_bs path to the data folder where the baseline scenarios are stored
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate discount rate
#' @param pattern regular exression indicating which files should be processed 
#' @param CoresNumber either number of cores or function detectCores()
#'
#' @return data frame of impacts of floods
#'
#' @example calcParallel_impacts(data.path_fl = file.path(data.path, "M3"), data.path_bs = file.path(data.path, "baseline", "M3"), ticks.stop = 120, disc.rate = 0.05, pattern = "m3\\w*conf1{1}[^0-9]\\w", CoresNumber = 4)
#' @example calcParallel_impacts(data.path_fl = file.path(data.path, "M3"), data.path_bs = file.path(data.path, "baseline", "M3"), ticks.stop = 120, disc.rate = 0.05, pattern = "m3\\w*conf1{1}[^0-9]\\w", CoresNumber = detectCores())
#'
#' @author David Nortes Martinez
#' @date 31/07/2017
calcParallel_impacts <- function(data.path_fl, data.path_bs, ticks.stop, disc.rate, pattern, CoresNumber, data.save){
  #~~~~ Checking whether required libraries are installed (and instaling them if so) and reading them
  check.package("gtools")
  check.package("doParallel")
  require(gtools)
  require(doParallel)
  
  #~~~~ setting up cluster
  cl <- makeCluster(CoresNumber)
  registerDoParallel(cl)
  getDoParWorkers()
  
  #~~~~ firt step: process baseline
  datafilelist <- mixedsort(dir(path = data.path_bs, pattern = pattern, full.names = F, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  baseline <- NULL
  for (i in seq_along(datafilelist)){
    temp <- read.csv(file.path(data.path_bs, datafilelist[i]), header = TRUE)
    farm_list <- unique(temp$agent)
    temp <- WineMakingComp(temp)
    aux <- foreach(j=seq_along(farm_list), .combine = rbind, .export = "data.manip") %dopar% {
            x <- data.manip(df = temp[temp$agent %in% farm_list[j], ],
                            floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                            config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                            season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                            ticks.stop = ticks.stop,
                            disc.rate = disc.rate)
            return(x)                  
      }
    baseline <- rbind(baseline, aux)
    }
  baseline <- data.frame(baseline, row.names = NULL)
  
  #~~~~ second step: process flood scenarios and calculate impacts
  datafilelist <- mixedsort(dir(path = data.path_fl, pattern = pattern, full.names = F, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  impact <- NULL
  for (i in seq_along(datafilelist)){
    if (!file.size(file.path(data.path_fl, datafilelist[i])) == 0) {
    temp <- read.csv(file.path(data.path_fl, datafilelist[i]), header = TRUE)
    temp <- WineMakingComp(temp)
    aux <- foreach(j=seq_along(farm_list), .combine = rbind, .export = "data.manip") %dopar% {
            #~~~~ Data manipulation for each farm
            x <- data.manip(df = temp[temp$agent %in% farm_list[j], ],
                            floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                            config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                            season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                            ticks.stop = ticks.stop,
                            disc.rate = disc.rate)
            return(x)                  
    }
    aux <- data.frame(aux, row.names = NULL)
    aux1 <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
            #~~~~ Search for its own baseline
            bs_x <- baseline[baseline$farm %in% farm_list[j] 
                             & baseline$flood.ext %in% unique(temp[temp$agent %in% farm_list[j], ]$flood) 
                             & baseline$dist.ct %in% unique(temp[temp$agent %in% farm_list[j], ]$dist.ct), ]
            
            #~~~~ impact calculation: flood - baseline
            imp_x <- sweep(aux[aux$farm %in% farm_list[j],-c(1,22:24)], 2, as.matrix(bs_x[1,-c(1,22:24)]))

            return(imp_x)                  
    }
    aux$rep = gsub(pattern  = '.*_rep(\\d*).*', '\\1', x = datafilelist[i])
    aux$winpos = wintac_finder(datafilelist[i])[[1]]
    aux$tactic = wintac_finder(datafilelist[i])[[2]]
    saveRDS(aux, file = paste(data.save,"/imp", sub(".csv","",datafilelist[i]), ".rds", sep = ""), compress = TRUE)
    impact <- rbind(impact, aux)
    }
  }
  impact <- data.frame(impact, row.names = NULL)
  impact$HLPD.LT = - impact$HLPD.LT
  impact[is.na(impact)] <- 0
  impact$WCV.ST = impact$WCST.CS + impact$WCST.CO
  impact$WCV.LT = impact$WCLT.CS + impact$WCLT.CO
  impact$TDdirect.ST = impact$WCST + impact$DFB + impact$SD + impact$PD + impact$PR + impact$HLF + impact$VCA.ST - impact$OUTS
  impact$TDindirect.ST = impact$DWB + impact$OUTS + impact$HLDBW
  impact$TDdirect.LT = impact$WCLT + impact$VCA.LT
  impact$TDindirect.LT = impact$HLPD.LT
  impact$TD.ST = impact$TDdirect.ST + impact$TDindirect.ST
  impact$TD.LT = impact$TDdirect.LT + impact$TDindirect.LT
  impact$TD = impact$TD.ST + impact$TD.LT 
  impact$fseason[impact$fseason %in% 1] <- "spring"
  impact$fseason[impact$fseason %in% 2] <- "summer"
  impact$fseason[impact$fseason %in% 3] <- "autumn"
  impact$fseason[impact$fseason %in% 4] <- "winter"
  
  #~~~~ Stop cluster
  stopCluster(cl)
  
  #~~~ Return results
  return(impact)
}


#' Function to calculate the impacts of a flood. M2 branch
#'
#' @param data.path_fl 
#' @param data.path_bs 
#' @param ticks.stop number of ticks allowed in each simulation  
#' @param disc.rate
#'
#' @return data frame of impacts of floods
#'
#' @example calcParallel_impacts(data.path_fl = file.path(data.path, "M2"), data.path_bs = file.path(data.path, "baseline", "M2"), ticks.stop = 120, disc.rate = 0.05)
#'
#' @author David Nortes Martinez
#' @date 31/07/2017
calcParallel_impacts.m2 <- function(data.path_fl, data.path_bs, ticks.stop, disc.rate,pattern, CoresNumber, data.save){
  #~~~~ Checking whether required libraries are installed (and instaling them if so) and reading them
  check.package("gtools")
  check.package("doParallel")
  require(gtools)
  require(doParallel)
  
  #~~~~ setting up cluster
  cl <- makeCluster(CoresNumber)
  registerDoParallel(cl)
  getDoParWorkers()
  
  #~~~~ firt step: process baseline
  datafilelist <- mixedsort(dir(path = data.path_bs, pattern = pattern, full.names = F, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  baseline <- NULL
  for (i in seq_along(datafilelist)){
    temp <- read.csv(file.path(data.path_bs, datafilelist[i]), header = TRUE)
    farm_list <- unique(temp$agent)
    #temp <- WineMakingComp(temp)
    aux <- foreach(j=seq_along(farm_list), .combine = rbind, .export = "data.manip.m2") %dopar% {
      x <- data.manip.m2(df = temp[temp$agent %in% farm_list[j], ],
                         floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                         config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                         season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                         ticks.stop = ticks.stop,
                         disc.rate = disc.rate)
      return(x)                  
    }
    baseline <- rbind(baseline, aux)
  }
  baseline <- data.frame(baseline, row.names = NULL)
  
  #~~~~ second step: process flood scenarios and calculate impacts
  datafilelist <- mixedsort(dir(path = data.path_fl, pattern = pattern, full.names = F, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  impact <- NULL
  for (i in seq_along(datafilelist)){
    if (!file.size(file.path(data.path_fl, datafilelist[i])) == 0) {
    temp <- read.csv(file.path(data.path_fl, datafilelist[i]), header = TRUE)
    #temp <- WineMakingComp(temp)
    aux <- foreach(j=seq_along(farm_list), .combine = rbind, .export = "data.manip.m2") %dopar% {
      #~~~~ Data manipulation for each farm
      x <- data.manip.m2(df = temp[temp$agent %in% farm_list[j], ],
                         floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                         config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                         season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                         ticks.stop = ticks.stop,
                         disc.rate = disc.rate)
      return(x)                  
    }
    aux <- data.frame(aux, row.names = NULL)
    aux1 <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
      #~~~~ Search for its own baseline
      bs_x <- baseline[baseline$farm %in% farm_list[j] 
                       & baseline$flood.ext %in% unique(temp[temp$agent %in% farm_list[j], ]$flood) 
                       & baseline$dist.ct %in% unique(temp[temp$agent %in% farm_list[j], ]$dist.ct), ]
      
      #~~~~ impact calculation: flood - baseline
      imp_x <- sweep(aux[aux$farm %in% farm_list[j],-c(1,14:16)], 2, as.matrix(bs_x[1,-c(1,14:16)]))
      
      return(imp_x)                  
    }
    aux$rep = gsub(pattern  = '.*_rep(\\d*).*', '\\1', x = datafilelist[i])
    aux$winpos = wintac_finder(datafilelist[i])[[1]]
    aux$tactic = wintac_finder(datafilelist[i])[[2]]
    saveRDS(aux, file = paste(data.save,"/imp", sub(".csv","",datafilelist[i]), ".rds", sep = ""), compress = TRUE)
    impact <- rbind(impact, aux)
    }
  }
  
  #~~~~ Stop cluster
  stopCluster(cl)
  
  #~~~ Return results
  return(impact)
}


#' Function to calculate the impacts of a flood. M1 branch
#'
#' @param data.path_fl 
#' @param data.path_bs 
#' @param ticks.stop number of ticks allowed in each simulation  
#' @param disc.rate
#'
#' @return data frame of impacts of floods
#'
#' @example calcParallel_impacts(data.path_fl = file.path(data.path, "M1"), data.path_bs = file.path(data.path, "baseline", "M1"), ticks.stop = 120, disc.rate = 0.05)
#'
#' @author David Nortes Martinez
#' @date 31/07/2017
calcParallel_impacts.m1 <- function(data.path_fl, data.path_bs, ticks.stop, disc.rate, pattern, CoresNumber, data.save){
  #~~~~ Checking whether required libraries are installed (and instaling them if so) and reading them
  check.package("gtools")
  check.package("doParallel")
  require(gtools)
  require(doParallel)
  
  #~~~~ setting up cluster
  cl <- makeCluster(CoresNumber)
  registerDoParallel(cl)
  getDoParWorkers()
  
  #~~~~ firt step: process baseline
  datafilelist <- mixedsort(dir(path = data.path_bs, pattern = pattern, full.names = F, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  baseline <- NULL
  for (i in seq_along(datafilelist)){
    temp <- read.csv(file.path(data.path_bs, datafilelist[i]), header = TRUE)
    farm_list <- unique(temp$agent)
    #temp <- WineMakingComp(temp)
    aux <- foreach(j=seq_along(farm_list), .combine = rbind, .export = "data.manip.m1") %dopar% {
      x <- data.manip.m1(df = temp[temp$agent %in% farm_list[j], ],
                         floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                         config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                         season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                         ticks.stop = ticks.stop,
                         disc.rate = disc.rate)
      return(x)                  
    }
    baseline <- rbind(baseline, aux)
  }
  baseline <- data.frame(baseline, row.names = NULL)
  
  #~~~~ second step: process flood scenarios and calculate impacts
  datafilelist <- mixedsort(dir(path = data.path_fl, pattern = pattern, full.names = F, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  impact <- NULL
  for (i in seq_along(datafilelist)){
    if (!file.size(file.path(data.path_fl, datafilelist[i])) == 0) {
    temp <- read.csv(file.path(data.path_fl, datafilelist[i]), header = TRUE)
    #temp <- WineMakingComp(temp)
    aux <- foreach(j=seq_along(farm_list), .combine = rbind, .export = "data.manip.m1") %dopar% {
      #~~~~ Data manipulation for each farm
      x <- data.manip.m1(df = temp[temp$agent %in% farm_list[j], ],
                         floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                         config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                         season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                         ticks.stop = ticks.stop,
                         disc.rate = disc.rate)
      return(x)                  
    }
    aux <- data.frame(aux, row.names = NULL)
    aux1 <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
      #~~~~ Search for its own baseline
      bs_x <- baseline[baseline$farm %in% farm_list[j] 
                       & baseline$flood.ext %in% unique(temp[temp$agent %in% farm_list[j], ]$flood) 
                       & baseline$dist.ct %in% unique(temp[temp$agent %in% farm_list[j], ]$dist.ct), ]
      
      #~~~~ impact calculation: flood - baseline
      imp_x <- sweep(aux[aux$farm %in% farm_list[j],-c(1,15:17)], 2, as.matrix(bs_x[1,-c(1,15:17)]))
      
      return(imp_x)                  
    }
    aux$rep = gsub(pattern  = '.*_rep(\\d*).*', '\\1', x = datafilelist[i])
    aux$winpos = wintac_finder(datafilelist[i])[[1]]
    aux$tactic = wintac_finder(datafilelist[i])[[2]]
    saveRDS(aux, file = paste(data.save,"/imp", sub(".csv","",datafilelist[i]), ".rds", sep = ""), compress = TRUE)
    impact <- rbind(impact, aux)
    }
  }
  
  #~~~~ Stop cluster
  stopCluster(cl)
  
  #~~~ Return results
  return(impact)
}


#' Function to calculate the different hypothetical when information is missing. M1 branch
#'
#' @param df 
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate
#' @param 
#'
#' @return data frame of impacts of floods
#'
#' @example calcParallel_impacts(data.path_fl = file.path(data.path, "M1"), data.path_bs = file.path(data.path, "baseline", "M1"), ticks.stop = 120, disc.rate = 0.05)
#'
#' @author David Nortes Martinez
#' @date 31/07/2017
m1h1_safe<- function(df, ticks.stop, disc.rate){
  #~~~~ xcor coop = 50
  #~~~~~~~Calculus of H1 and H2
  periods <- c(1:(ticks.stop/4))
  discount.factors <- (1+disc.rate)^(1-periods)
  coop.rep = 3480000
  levels(df$fseason) <- c("autumn", "spring", "summer", "winter")
  if ("rep" %in% names(df)){
    nbCoops <-  expand.grid(flood.ext = levels(as.factor(df$flood.ext)),
                            dist = levels(df$dist.ct),
                            fseason = levels(df$fseason),
                            rep = levels(df$rep),
                            winpos = levels(df$winpos),
                            tactic = levels(df$tactic))
  } else {
    nbCoops <-  expand.grid(flood.ext = levels(as.factor(df$flood.ext)),
                            dist = levels(df$dist.ct),
                            fseason = levels(df$fseason),
                            winpos = levels(df$winpos),
                            tactic = levels(df$tactic))
  }
  nbCoops$nbCoops = 0
  nbCoops$DWB = nbCoops$nbCoops * coop.rep
  nbCoops$HLDBW.H1 = nbCoops$nbCoops * 2822400
  nbCoops$WCST.H1 = nbCoops$nbCoops * (-564480)
  nbCoops$WCLT.H1 = 0
  #~~~~~~~~~~~~~~~~~~~~~~~ Hypothesis H1
  df$HLPD.LT = -df$HLPD.LT
  if ("rep" %in% names(df)){
    df <- merge(df, nbCoops, by = c(1:6), all = TRUE, sort = TRUE)
  } else {
    df <- merge(df, nbCoops, by = c(1:5), all = TRUE, sort = TRUE)
  }
  
  df$TDindirect.ST = df$DWB + df$HLDBW
  df$TDdirect.ST = df$DFB +df$SD + df$PD + df$PR + df$HLF + df$VCAH1.ST + df$WCST
  df$TDdirect.LT =  df$VCAH1.LT + df$WCLT
  df$TDindirect.LT = df$HLPD.LT
  df$TD.ST = df$TDdirect.ST + df$TDindirect.ST
  df$TD.LT = df$TDdirect.LT + df$TDindirect.LT
  df$TD = df$TD.ST + df$TD.LT 
  
  #df[is.na(df)] <- 0
  df$nbCoops = NULL
  colnames(df)[colnames(df) %in% c('DWB', 'HLDBW.H1', 'WCST.H1', 'WCLT.H1')] <- c('DWB', 'HLDBW', 'WCST', 'WCLT')
  colnames(df)[colnames(df) == 'VCAH1.ST'] <- 'VCA.ST'
  colnames(df)[colnames(df) == 'VCAH1.LT'] <- 'VCA.LT'
  df$VCAH2.ST = NULL
  df$VCAH2.LT = NULL
  
  return(df)
}


#' Function to calculate the different hypothetical when information is missing. M1 branch
#'
#' @param df 
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate
#' @param 
#'
#' @return data frame of impacts of floods
#'
#' @example calcParallel_impacts(data.path_fl = file.path(data.path, "M1"), data.path_bs = file.path(data.path, "baseline", "M1"), ticks.stop = 120, disc.rate = 0.05)
#'
#' @author David Nortes Martinez
#' @date 31/07/2017
m1h1_prone<- function(df, ticks.stop, disc.rate){
  #~~~~ xcor coop = 50
  #~~~~~~~Calculus of H1 and H2
  periods <- c(1:(ticks.stop/4))
  discount.factors <- (1+disc.rate)^(1-periods)
  coop.rep = 3480000
  levels(df$fseason) <- c("autumn", "spring", "summer", "winter")
  if ("rep" %in% names(df)){
    nbCoops <-  expand.grid(flood.ext = levels(as.factor(df$flood.ext)),
                            dist = levels(df$dist.ct),
                            fseason = levels(df$fseason),
                            rep = levels(df$rep),
                            winpos = levels(df$winpos),
                            tactic = levels(df$tactic))
  } else {
    nbCoops <-  expand.grid(flood.ext = levels(as.factor(df$flood.ext)),
                            dist = levels(df$dist.ct),
                            fseason = levels(df$fseason),
                            winpos = levels(df$winpos),
                            tactic = levels(df$tactic))
  }
  nbCoops$nbCoops = 1
  nbCoops$DWB = nbCoops$nbCoops * coop.rep
  nbCoops$HLDBW.H1 = nbCoops$nbCoops * 2822400
  nbCoops$HLDBW.H1[nbCoops$fseason %in% c("summer")] = 0
  nbCoops$WCST.H1 = nbCoops$nbCoops * (-564480)
  nbCoops$WCST.H1[nbCoops$fseason %in% c("summer", "autumn", "spring")] = 0
  nbCoops$WCLT.H1 = 0
  #~~~~~~~~~~~~~~~~~~~~~~~ Hypothesis H1
  df$HLPD.LT = -df$HLPD.LT
  if ("rep" %in% names(df)){
    df <- merge(df, nbCoops, by = c(1:6), all = TRUE, sort = TRUE)
  } else {
    df <- merge(df, nbCoops, by = c(1:5), all = TRUE, sort = TRUE)
  }
  
  df$TDindirect.ST = df$DWB + df$HLDBW
  df$TDdirect.ST = df$DFB +df$SD + df$PD + df$PR + df$HLF + df$VCAH1.ST + df$WCST
  df$TDdirect.LT =  df$VCAH1.LT + df$WCLT
  df$TDindirect.LT = df$HLPD.LT
  df$TD.ST = df$TDdirect.ST + df$TDindirect.ST
  df$TD.LT = df$TDdirect.LT + df$TDindirect.LT
  df$TD = df$TD.ST + df$TD.LT 
  
  #df[is.na(df)] <- 0
  df$nbCoops <- NULL
  colnames(df)[colnames(df) %in% c('DWB', 'HLDBW.H1', 'WCST.H1', 'WCLT.H1')] <- c('DWB', 'HLDBW', 'WCST', 'WCLT')
  colnames(df)[colnames(df) == 'VCAH1.ST'] <- 'VCA.ST'
  colnames(df)[colnames(df) == 'VCAH1.LT'] <- 'VCA.LT'
  df$VCAH2.ST = NULL
  df$VCAH2.LT = NULL
  
  return(df)
}


#' Function to calculate the different hypothetical when information is missing. M2 branch
#'
#' @param df 
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate
#' @param 
#'
#' @return data frame of impacts of floods
#'
#' @example calcParallel_impacts(data.path_fl = file.path(data.path, "M1"), data.path_bs = file.path(data.path, "baseline", "M1"), ticks.stop = 120, disc.rate = 0.05)
#'
#' @author David Nortes Martinez
#' @date 31/07/2017
m2h1_safe<- function(df, ticks.stop, disc.rate){
  #~~~~ xcor coop = 50
  #~~~~~~~Calculus of H1 and H2
  periods <- c(1:(ticks.stop/4))
  discount.factors <- (1+disc.rate)^(1-periods)
  coop.rep = 3480000
  levels(df$fseason) <- c("autumn", "spring", "summer", "winter")
  if ("rep" %in% names(df)){
    nbCoops <-  expand.grid(flood.ext = levels(as.factor(df$flood.ext)),
                            dist.ct = levels(df$dist.ct),
                            fseason = levels(df$fseason),
                            rep = levels(df$rep),
                            winpos = levels(df$winpos),
                            tactic = levels(df$tactic))
  } else {
    nbCoops <-  expand.grid(flood.ext = levels(as.factor(df$flood.ext)),
                            dist.ct = levels(df$dist.ct),
                            fseason = levels(df$fseason),
                            winpos = levels(df$winpos),
                            tactic = levels(df$tactic))
  }
  nbCoops$nbCoops = 0
  nbCoops$DWB = nbCoops$nbCoops * coop.rep
  nbCoops$HLDBW.H1 = nbCoops$nbCoops * 2822400
  nbCoops$WCST.H1 = nbCoops$nbCoops * (-564480)
  nbCoops$WCLT.H1 = 0
  #~~~~~~~~~~~~~~~~~~~~~~~ Hypothesis H1
  df$HLPD.LT = -df$HLPD.LT
  if ("rep" %in% names(df)){
    df <- merge(df, nbCoops, by = c(1:6), all = TRUE, sort = TRUE)
  } else {
    df <- merge(df, nbCoops, by = c(1:5), all = TRUE, sort = TRUE)
  }
  
  df$nbCoops = NULL
  colnames(df)[colnames(df) %in% c('DWB', 'HLDBW.H1', 'WCST.H1', 'WCLT.H1')] <- c('DWB', 'HLDBW', 'WCST', 'WCLT')
  df$TDindirect.ST = df$DWB + df$HLDBW
  df$TDdirect.ST = df$DFB +df$SD + df$PD + df$PR + df$HLF + df$VCA.ST + df$WCST
  df$TDdirect.LT =  df$VCA.LT + df$WCLT
  df$TDindirect.LT = df$HLPD.LT
  df$TD.ST = df$TDdirect.ST + df$TDindirect.ST
  df$TD.LT = df$TDdirect.LT + df$TDindirect.LT
  df$TD = df$TD.ST + df$TD.LT 
  #df[is.na(df)] <- 0
  
  return(df)
}


#' Function to calculate the different hypothetical when information is missing. M2 branch
#'
#' @param df 
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate
#' @param 
#'
#' @return data frame of impacts of floods
#'
#' @example calcParallel_impacts(data.path_fl = file.path(data.path, "M1"), data.path_bs = file.path(data.path, "baseline", "M1"), ticks.stop = 120, disc.rate = 0.05)
#'
#' @author David Nortes Martinez
#' @date 31/07/2017
m2h1_prone<- function(df, ticks.stop, disc.rate){
  #~~~~ xcor coop = 50
  #~~~~~~~Calculus of H1 and H2
  periods <- c(1:(ticks.stop/4))
  discount.factors <- (1+disc.rate)^(1-periods)
  coop.rep = 3480000
  levels(df$fseason) <- c("autumn", "spring", "summer", "winter")
  if ("rep" %in% names(df)){
    nbCoops <-  expand.grid(flood.ext = levels(as.factor(df$flood.ext)),
                            dist.ct = levels(df$dist.ct),
                            fseason = levels(df$fseason),
                            rep = levels(df$rep),
                            winpos = levels(df$winpos),
                            tactic = levels(df$tactic))
  } else {
    nbCoops <-  expand.grid(flood.ext = levels(as.factor(df$flood.ext)),
                            dist.ct = levels(df$dist.ct),
                            fseason = levels(df$fseason),
                            winpos = levels(df$winpos),
                            tactic = levels(df$tactic))
  }
  nbCoops$nbCoops = 1
  nbCoops$DWB = nbCoops$nbCoops * coop.rep
  nbCoops$HLDBW.H1 = nbCoops$nbCoops * 2822400
  nbCoops$HLDBW.H1[nbCoops$fseason %in% c("summer")] = 0
  nbCoops$WCST.H1 = nbCoops$nbCoops * (-564480)
  nbCoops$WCST.H1[nbCoops$fseason %in% c("summer", "autumn", "spring")] = 0
  nbCoops$WCLT.H1 = 0
  #~~~~~~~~~~~~~~~~~~~~~~~ Hypothesis H1
  df$HLPD.LT = -df$HLPD.LT
  if ("rep" %in% names(df)){
    df <- merge(df, nbCoops, by = c(1:6), all = TRUE, sort = TRUE)
  } else {
    df <- merge(df, nbCoops, by = c(1:5), all = TRUE, sort = TRUE)
  }
  
  df$nbCoops <- NULL
  colnames(df)[colnames(df) %in% c('DWB', 'HLDBW.H1', 'WCST.H1', 'WCLT.H1')] <- c('DWB', 'HLDBW', 'WCST', 'WCLT')
  df$TDindirect.ST = df$DWB + df$HLDBW
  df$TDdirect.ST = df$DFB +df$SD + df$PD + df$PR + df$HLF + df$VCA.ST + df$WCST
  df$TDdirect.LT =  df$VCA.LT + df$WCLT
  df$TDindirect.LT = df$HLPD.LT
  df$TD.ST = df$TDdirect.ST + df$TDindirect.ST
  df$TD.LT = df$TDdirect.LT + df$TDindirect.LT
  df$TD = df$TD.ST + df$TD.LT 
  #df[is.na(df)] <- 0
  
  return(df)
}

#' Function to calculate the different damages in the time spans chosen for the model. Parallel-ready
#'
#' @param data.path_fl path to the data folder where the results of the fllod simulations are stored
#' @param data.save path to folder where to store the files returned by the function.
#' @param DBname string containing the name to be given to the database once all data files are bound together
#' @param model string with one of the following values: "m1" for a model with no interaction whatsoever; "m2" for a model with partial interaction; "m3" for a model with full interaction   
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate discount rate
#' @param pattern regular exression indicating which files should be processed 
#' @param CoresNumber either number of cores or function detectCores()
#'
#' @return two files. One is individualized by each input file given to the functions and contains the simulation data for each indicator and time span. The second one is a single data base with all the results calculated by the function with all the files passed. Both are stored as RDS files
#' 
#' @example calcTimeScale(data.path_fl = "~/home/M3", data.save = "~/home/M3", DBname = "m3_conf1", model = "m3", ticks.stop = 120, disc.rate = 0.05, pattern = "m3\\w*conf1{1}[^0-9]\\w", CoresNumber = 4)
#' @example calcTimeScale(data.path_fl = file.path(data.path, "M3"), data.save = file.path(data.path, "M3"), DBname = "m3_conf1", model = "m3", ticks.stop = 120, disc.rate = 0.05, pattern = "m3\\w*conf1{1}[^0-9]\\w", CoresNumber = detectCores())
#' @example calcTimeScale(data.path_fl = file.path(data.path, "M3"), data.save = file.path(data.path, "M3"), DBname = "m3_conf1", model = "m3", ticks.stop = 120, disc.rate = 0.05, pattern = "m3\\w*conf1{1}[^0-9]\\w", CoresNumber = (detectCores()-1))
#'
#' @author David Nortes Martinez
#' @date 03/08/2017
calcTimeScale <- function(data.path_fl, data.save, DBname, model, ticks.stop, disc.rate, pattern, CoresNumber){
  #~~~~ Checking whether required libraries are installed (and instaling them if so) and reading them
  check.package("gtools")
  check.package("doParallel")
  require(gtools)
  require(doParallel)
  
  #~~~~ setting up cluster
  cl <- makeCluster(CoresNumber)
  registerDoParallel(cl)
  getDoParWorkers()
  
  #~~~~ process flood scenarios and calculate time spans for the indicators. files are chosen according to the regular espression provided
  datafilelist <- mixedsort(dir(path = data.path_fl, pattern = pattern, full.names = F, include.dirs = FALSE)) # creates the list of all the rds files in the directory
  auxDB <- NULL
  clusterExport(cl = cl, c("data.manip.m1", "data.manip.m2", "data.manip"), envir = .GlobalEnv)
  for (i in seq_along(datafilelist)){
    if (!file.size(file.path(data.path_fl, datafilelist[i])) == 0) {
      if(grepl(".rds", datafilelist[i])){
          temp <- readRDS(file = file.path(data.path_fl, datafilelist[i]))
        } else {
          temp <- read.csv(file.path(data.path_fl, datafilelist[i]), header = TRUE)
        }
      
      farm_list <- unique(temp$agent)
      #~~~~ choosing correct function depending on the value of parameter "model"
      if (model =="m1"){
        aux <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
          #~~~~ Data manipulation for each farm
          x <- data.manip.m1(df = temp[temp$agent %in% farm_list[j], ],
                             floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                             config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                             season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                             ticks.stop = ticks.stop,
                             disc.rate = disc.rate)
          return(x)                  
        }
      }
      
      if (model =="m2"){
        aux <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
          #~~~~ Data manipulation for each farm
          x <- data.manip.m2(df = temp[temp$agent %in% farm_list[j], ],
                             floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                             config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                             season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                             ticks.stop = ticks.stop,
                             disc.rate = disc.rate)
          return(x)                  
        }
      }
      
      if (model =="m3"){
        temp <- WineMakingComp(temp)
        aux <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
          #~~~~ Data manipulation for each farm
          x <- data.manip(df = temp[temp$agent %in% farm_list[j], ],
                          floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                          config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                          season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                          ticks.stop = ticks.stop,
                          disc.rate = disc.rate)
          return(x)                  
        } 
      }
      
      #~~~~ converting the matrix obtained with the foreach lood and adding ID columns
      aux <- data.frame(aux, row.names = NULL)
      aux$rep = gsub(pattern  = '.*_rep(\\d*).*', '\\1', x = datafilelist[i])
      aux$winpos = wintac_finder(datafilelist[i])[[1]]
      aux$tactic = wintac_finder(datafilelist[i])[[2]]
      #~~~~save individual auxiliar file of each simulation included in datafilelist
      saveRDS(aux, file = file.path(data.save, paste("aux_", sub(".csv","",datafilelist[i]), sep = "")), compress = TRUE)
      #building grouped database
      auxDB <- rbind(auxDB, aux)
    }
  }
  auxDB <- data.frame(auxDB, row.names = NULL)
  saveRDS(auxDB, 
          file = file.path(data.save, paste(DBname, ".rds", sep = "")),
          compress = TRUE)
  #~~~~ Stop cluster
  stopCluster(cl)
}


#' Function to calculate the different damages in the time spans chosen for the model. Parallel-ready
#'
#' @param DBname name of the database with the auxiliary collection of operations made over each simulation by the function calcTimeScale
#' @param DB.path path to database in DBname
#' @param data.save path to folder where to store the files returned by the function.
#' @param IMPname NAme to store the result given by the function
#' @param data.path_bs path to the data folder where the baseline scenarios for flood simulations are stored
#' @param model string with one of the following values: "m1" for a model with no interaction whatsoever; "m2" for a model with partial interaction; "m3" for a model with full interaction   
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate discount rate
#' @param pattern regular exression indicating which files should be processed 
#' @param CoresNumber either number of cores or function detectCores()
#'
#' @return data base of impacts
#'
#' @example calcImpacts(DBname = "m3_conf1", DB.path = "/home/davidnortes/Desktop/test_func", data.save = "/home/davidnortes/Desktop/test_func", IMPname = "imp_m3_conf1", data.path_bs = "/home/davidnortes/Desktop/output/baseline/M3", model = "m3", ticks.stop = 120, disc.rate = 0.05, pattern = "m3\\w*conf1{1}[^0-9]\\w", CoresNumber = detectCores())
#'
#' @author David Nortes Martinez
#' @date 03/08/2017
calcImpacts <- function(DBname, DB.path, data.save, IMPname, model, data.path_bs, ticks.stop, disc.rate, pattern, CoresNumber){
  #~~~~ Checking whether required libraries are installed (and instaling them if so) and reading them
  check.package("gtools")
  check.package("doParallel")
  require(gtools)
  require(doParallel)
  
  #~~~~ setting up cluster
  cl <- makeCluster(CoresNumber)
  registerDoParallel(cl)
  getDoParWorkers()
  
  #~~~~ firt step: process baseline
  datafilelist <- mixedsort(dir(path = data.path_bs, pattern = pattern, full.names = FALSE, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  baseline <- NULL
  clusterExport(cl = cl, c("data.manip"), envir = .GlobalEnv)
  for (i in seq_along(datafilelist)){
    temp <- read.csv(file.path(data.path_bs, datafilelist[i]), header = TRUE)
    farm_list <- unique(temp$agent)
    #~~~~ choosing correct function depending on the value of parameter "model
    
      temp <- WineMakingComp(temp)
      aux <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
        x <- data.manip(df = temp[temp$agent %in% farm_list[j], ],
                        floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                        config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                        season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                        ticks.stop = ticks.stop,
                        disc.rate = disc.rate)
        return(x)
      }
    baseline <- rbind(baseline, aux)
  }
  baseline <- data.frame(baseline, row.names = NULL)
  
  #~~~~ second step: process flood scenarios and calculate impacts
  db <- readRDS(file.path(DB.path, paste(DBname,".rds",sep = "")))
  aux <- foreach(j=1:nrow(db), .combine = rbind) %dopar% { #, .export = "model"
    #~~~~ Search for its own baseline
    bs_x <- baseline[baseline$Farm %in% db$Farm[j] 
                     & baseline$dist.ct %in% db$dist.ct[j], ]
    #~~~~ impact calculation: flood - baseline
      imp_x <- sweep(db[j,-c(1,22:27)], 2, as.matrix(bs_x[1,-c(1,22:24)]))
    return(imp_x)                  
  }

    aux <- cbind(db[ ,c(1,22:27)], aux)
    aux <- data.frame(aux, row.names = NULL)
    aux[is.na(aux)] <- 0
    aux$HLPD.LT = - aux$HLPD.LT
    aux$WCV.ST = aux$WCST.CS + aux$WCST.CO
    aux$WCV.LT = aux$WCLT.CS + aux$WCLT.CO
    aux$TDdirect.ST = aux$WCST + aux$DFB + aux$SD + aux$PD + aux$PR + aux$HLF + aux$VCA.ST - aux$OUTS
    aux$TDindirect.ST = aux$DWB + aux$OUTS + aux$HLDBW
    aux$TDdirect.LT = aux$WCLT + aux$VCA.LT
    aux$TDindirect.LT = aux$HLPD.LT
    aux$TD.ST = aux$TDdirect.ST + aux$TDindirect.ST
    aux$TD.LT = aux$TDdirect.LT + aux$TDindirect.LT
    aux$TD = aux$TD.ST + aux$TD.LT 
    #aux$fseason[aux$fseason %in% 1] <- "spring"
    #aux$fseason[aux$fseason %in% 2] <- "summer"
    #aux$fseason[aux$fseason %in% 3] <- "autumn"
    #aux$fseason[aux$fseason %in% 4] <- "winter"
  #~~~~ Stop cluster
  stopCluster(cl)
  
  #~~~ Return results
  saveRDS(aux, 
          file = file.path(data.save, paste(IMPname, ".rds", sep = "")),
          compress = TRUE)
  #return(aux)
}


#' Function to calculate the different damages in the time spans chosen for the model. Parallel-ready
#'
#' @param DBname name of the database with the auxiliary collection of operations made over each simulation by the function calcTimeScale
#' @param DB.path path to database in DBname
#' @param data.save path to folder where to store the files returned by the function.
#' @param IMPname NAme to store the result given by the function
#' @param data.path_bs path to the data folder where the baseline scenarios for flood simulations are stored
#' @param model string with one of the following values: "m1" for a model with no interaction whatsoever; "m2" for a model with partial interaction; "m3" for a model with full interaction   
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate discount rate
#' @param pattern regular exression indicating which files should be processed 
#' @param CoresNumber either number of cores or function detectCores()
#'
#' @return data base of impacts
#'
#' @example calcImpacts(DBname = "m3_conf1", DB.path = "/home/davidnortes/Desktop/test_func", data.save = "/home/davidnortes/Desktop/test_func", IMPname = "imp_m3_conf1", data.path_bs = "/home/davidnortes/Desktop/output/baseline/M3", model = "m3", ticks.stop = 120, disc.rate = 0.05, pattern = "m3\\w*conf1{1}[^0-9]\\w", CoresNumber = detectCores())
#'
#' @author David Nortes Martinez
#' @date 03/08/2017
calcImpacts.m2 <- function(DBname, DB.path, data.save, IMPname, model, data.path_bs, ticks.stop, disc.rate, pattern, CoresNumber){
  #~~~~ Checking whether required libraries are installed (and instaling them if so) and reading them
  check.package("gtools")
  check.package("doParallel")
  require(gtools)
  require(doParallel)
  
  #~~~~ setting up cluster
  cl <- makeCluster(CoresNumber)
  registerDoParallel(cl)
  getDoParWorkers()
  
  #~~~~ firt step: process baseline
  datafilelist <- mixedsort(dir(path = data.path_bs, pattern = pattern, full.names = FALSE, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  baseline <- NULL
  clusterExport(cl = cl, c("data.manip.m2"), envir = .GlobalEnv)
  for (i in seq_along(datafilelist)){
    temp <- read.csv(file.path(data.path_bs, datafilelist[i]), header = TRUE)
    farm_list <- unique(temp$agent)
    #~~~~ choosing correct function depending on the value of parameter "model
  
      aux <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
        x <- data.manip.m2(df = temp[temp$agent %in% farm_list[j], ],
                           floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                           config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                           season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                           ticks.stop = ticks.stop,
                           disc.rate = disc.rate)
        return(x)
      }
    baseline <- rbind(baseline, aux)
  }
  baseline <- data.frame(baseline, row.names = NULL)
  
  #~~~~ second step: process flood scenarios and calculate impacts
  db <- readRDS(file.path(DB.path, paste(DBname,".rds",sep = "")))
  aux <- foreach(j=1:nrow(db), .combine = rbind) %dopar% { #, .export = "model"
    #~~~~ Search for its own baseline
    bs_x <- baseline[baseline$Farm %in% db$Farm[j] 
                     & baseline$dist.ct %in% db$dist.ct[j], ]
    
    #~~~~ impact calculation: flood - baseline
    
      imp_x <- sweep(db[j,-c(1,14:19)], 2, as.matrix(bs_x[1,-c(1,14:16)]))
      
    return(imp_x)                  
  }
  
    aux <- cbind(db[ ,c(1,14:19)], aux)
    aux <- data.frame(aux, row.names = NULL)
  #~~~~ Stop cluster
  stopCluster(cl)
  
  #~~~ Return results
  saveRDS(aux, 
          file = file.path(data.save, paste(IMPname, ".rds", sep = "")),
          compress = TRUE)
  #return(aux)
}



#' Function to calculate the different damages in the time spans chosen for the model. Parallel-ready
#'
#' @param DBname name of the database with the auxiliary collection of operations made over each simulation by the function calcTimeScale
#' @param DB.path path to database in DBname
#' @param data.save path to folder where to store the files returned by the function.
#' @param IMPname NAme to store the result given by the function
#' @param data.path_bs path to the data folder where the baseline scenarios for flood simulations are stored
#' @param model string with one of the following values: "m1" for a model with no interaction whatsoever; "m2" for a model with partial interaction; "m3" for a model with full interaction   
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate discount rate
#' @param pattern regular exression indicating which files should be processed 
#' @param CoresNumber either number of cores or function detectCores()
#'
#' @return data base of impacts
#'
#' @example calcImpacts(DBname = "m3_conf1", DB.path = "/home/davidnortes/Desktop/test_func", data.save = "/home/davidnortes/Desktop/test_func", IMPname = "imp_m3_conf1", data.path_bs = "/home/davidnortes/Desktop/output/baseline/M3", model = "m3", ticks.stop = 120, disc.rate = 0.05, pattern = "m3\\w*conf1{1}[^0-9]\\w", CoresNumber = detectCores())
#'
#' @author David Nortes Martinez
#' @date 03/08/2017
calcImpacts.m1 <- function(DBname, DB.path, data.save, IMPname, model, data.path_bs, ticks.stop, disc.rate, pattern, CoresNumber){
  #~~~~ Checking whether required libraries are installed (and instaling them if so) and reading them
  check.package("gtools")
  check.package("doParallel")
  require(gtools)
  require(doParallel)
  
  #~~~~ setting up cluster
  cl <- makeCluster(CoresNumber)
  registerDoParallel(cl)
  getDoParWorkers()
  
  #~~~~ firt step: process baseline
  datafilelist <- mixedsort(dir(path = data.path_bs, pattern = pattern, full.names = FALSE, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  baseline <- NULL
  clusterExport(cl = cl, c("data.manip.m1"), envir = .GlobalEnv)
  for (i in seq_along(datafilelist)){
    temp <- read.csv(file.path(data.path_bs, datafilelist[i]), header = TRUE)
    farm_list <- unique(temp$agent)
    #~~~~ choosing correct function depending on the value of parameter "model
   
      aux <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
        x <- data.manip.m1(df = temp[temp$agent %in% farm_list[j], ],
                           floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                           config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                           season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                           ticks.stop = ticks.stop,
                           disc.rate = disc.rate)
        return(x)
      }
    
    baseline <- rbind(baseline, aux)
  }
  baseline <- data.frame(baseline, row.names = NULL)
  
  #~~~~ second step: process flood scenarios and calculate impacts
  db <- readRDS(file.path(DB.path, paste(DBname,".rds",sep = "")))
  aux <- foreach(j=1:nrow(db), .combine = rbind) %dopar% { #, .export = "model"
    #~~~~ Search for its own baseline
    bs_x <- baseline[baseline$Farm %in% db$Farm[j] 
                     & baseline$dist.ct %in% db$dist.ct[j], ]
    
    #~~~~ impact calculation: flood - baseline
      imp_x <- sweep(db[j,-c(1,15:20)], 2, as.matrix(bs_x[1,-c(1,15:17)]))
    
    return(imp_x)                  
  }
  
    aux <- cbind(db[ ,c(1,15:20)], aux)
    aux <- data.frame(aux, row.names = NULL)

  #~~~~ Stop cluster
  stopCluster(cl)
  
  #~~~ Return results
  saveRDS(aux, 
          file = file.path(data.save, paste(IMPname, ".rds", sep = "")),
          compress = TRUE)
  #return(aux)
}


#' Function to calculate the different damages in the time spans chosen for the model. Parallel-ready
#'
#' @param DBname name of the database with the auxiliary collection of operations made over each simulation by the function calcTimeScale
#' @param DB.path path to database in DBname
#' @param data.save path to folder where to store the files returned by the function.
#' @param IMPname NAme to store the result given by the function
#' @param data.path_bs path to the data folder where the baseline scenarios for flood simulations are stored
#' @param model string with one of the following values: "m1" for a model with no interaction whatsoever; "m2" for a model with partial interaction; "m3" for a model with full interaction   
#' @param ticks.stop number of ticks allowed in each simulation 
#' @param disc.rate discount rate
#' @param pattern regular exression indicating which files should be processed 
#' @param CoresNumber either number of cores or function detectCores()
#'
#' @return data base of impacts
#'
#' @example calcImpacts(DBname = "m3_conf1", DB.path = "/home/davidnortes/Desktop/test_func", data.save = "/home/davidnortes/Desktop/test_func", IMPname = "imp_m3_conf1", data.path_bs = "/home/davidnortes/Desktop/output/baseline/M3", model = "m3", ticks.stop = 120, disc.rate = 0.05, pattern = "m3\\w*conf1{1}[^0-9]\\w", CoresNumber = detectCores())
#'
#' @author David Nortes Martinez
#' @date 03/08/2017
calcImpacts.all <- function(DBname, DB.path, data.save, IMPname, model, data.path_bs, ticks.stop, disc.rate, pattern, CoresNumber){
  #~~~~ Checking whether required libraries are installed (and instaling them if so) and reading them
  check.package("gtools")
  check.package("doParallel")
  require(gtools)
  require(doParallel)
  
  #~~~~ setting up cluster
  cl <- makeCluster(CoresNumber)
  registerDoParallel(cl)
  getDoParWorkers()
  
  #~~~~ firt step: process baseline
  datafilelist <- mixedsort(dir(path = data.path_bs, pattern = pattern, full.names = FALSE, include.dirs = FALSE)) # creates the list of all the csv files in the directory
  baseline <- NULL
  clusterExport(cl = cl, c("data.manip.m1", "data.manip.m2", "data.manip"), envir = .GlobalEnv)
  for (i in seq_along(datafilelist)){
    temp <- read.csv(file.path(data.path_bs, datafilelist[i]), header = TRUE)
    farm_list <- unique(temp$agent)
    #~~~~ choosing correct function depending on the value of parameter "model
    if (model =="m1"){
      aux <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
        x <- data.manip.m1(df = temp[temp$agent %in% farm_list[j], ],
                           floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                           config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                           season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                           ticks.stop = ticks.stop,
                           disc.rate = disc.rate)
        return(x)
      }
    }
    
    if (model =="m2"){
      aux <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
        x <- data.manip.m2(df = temp[temp$agent %in% farm_list[j], ],
                           floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                           config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                           season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                           ticks.stop = ticks.stop,
                           disc.rate = disc.rate)
        return(x)
      }
    }
    
    if (model =="m3"){
      temp <- WineMakingComp(temp)
      aux <- foreach(j=seq_along(farm_list), .combine = rbind) %dopar% {
        x <- data.manip(df = temp[temp$agent %in% farm_list[j], ],
                        floodextent = unique(temp[temp$agent %in% farm_list[j], ]$flood),
                        config = unique(temp[temp$agent %in% farm_list[j], ]$dist.ct),
                        season = unique(temp[temp$agent %in% farm_list[j], ]$fseason),
                        ticks.stop = ticks.stop,
                        disc.rate = disc.rate)
        return(x)
      }
    }
    
    baseline <- rbind(baseline, aux)
  }
  baseline <- data.frame(baseline, row.names = NULL)
  
  #~~~~ second step: process flood scenarios and calculate impacts
  db <- readRDS(file.path(DB.path, paste(DBname,".rds",sep = "")))
  aux <- foreach(j=1:nrow(db), .combine = rbind) %dopar% { #, .export = "model"
    #~~~~ Search for its own baseline
    bs_x <- baseline[baseline$Farm %in% db$Farm[j] 
                     & baseline$dist.ct %in% db$dist.ct[j], ]
    
    #~~~~ impact calculation: flood - baseline
    if (model =="m1"){
      imp_x <- sweep(db[j,-c(1,15:20)], 2, as.matrix(bs_x[1,-c(1,15:17)]))
    }
    
    if (model =="m2"){
      imp_x <- sweep(db[j,-c(1,14:19)], 2, as.matrix(bs_x[1,-c(1,14:16)]))
    }
    
    if (model =="m3"){
      imp_x <- sweep(db[j,-c(1,22:27)], 2, as.matrix(bs_x[1,-c(1,22:24)]))
    }
    return(imp_x)                  
  }
  
  if (model =="m1"){
    aux <- cbind(db[ ,c(1,15:20)], aux)
    aux <- data.frame(aux, row.names = NULL)
  }
  
  if (model =="m2"){
    aux <- cbind(db[ ,c(1,14:19)], aux)
    aux <- data.frame(aux, row.names = NULL)
  }
  
  if (model =="m3"){
    aux <- cbind(db[ ,c(1,22:27)], aux)
    aux <- data.frame(aux, row.names = NULL)
    aux[is.na(aux)] <- 0
    aux$HLPD.LT = - aux$HLPD.LT
    aux$WCV.ST = aux$WCST.CS + aux$WCST.CO
    aux$WCV.LT = aux$WCLT.CS + aux$WCLT.CO
    aux$TDdirect.ST = aux$WCST + aux$DFB + aux$SD + aux$PD + aux$PR + aux$HLF + aux$VCA.ST - aux$OUTS
    aux$TDindirect.ST = aux$DWB + aux$OUTS + aux$HLDBW
    aux$TDdirect.LT = aux$WCLT + aux$VCA.LT
    aux$TDindirect.LT = aux$HLPD.LT
    aux$TD.ST = aux$TDdirect.ST + aux$TDindirect.ST
    aux$TD.LT = aux$TDdirect.LT + aux$TDindirect.LT
    aux$TD = aux$TD.ST + aux$TD.LT 
    aux$fseason[aux$fseason %in% 1] <- "spring"
    aux$fseason[aux$fseason %in% 2] <- "summer"
    aux$fseason[aux$fseason %in% 3] <- "autumn"
    aux$fseason[aux$fseason %in% 4] <- "winter"
  }
  #~~~~ Stop cluster
  stopCluster(cl)
  
  #~~~ Return results
  saveRDS(aux, 
          file = file.path(data.save, paste(IMPname, ".rds", sep = "")),
          compress = TRUE)
  #return(aux)
}


