

;;;EXTENSIONS;;;;;

;;;GLOBALS AND VARIABLES;;;;;;

globals [
  nbFarmPerCoop
  nbCoop
  nbLandPerFarm
  season               ;vector of seasons: {winter, spring, summer, autumn}
  current-season       ;(INTEGER) It takes values from a predefined vector of seasons: {winter, spring, summer, autumn}
  wine-prices          ;(FLOAT)
  productivity-ha
  soil-reparation-cost
  plant-replacement-cost
  farm-reparation-cost
  coop-reparation-cost
  farm-insured-cost
  coop-insured-cost
  year
  age-productive
  age-old
  input-data
  outsource-increment
  seasonal-operational-cost
  results
  ConseqCost
  ConseqHarv
  cropland-dest
]


breed [cooperatives cooperative]
breed [croplands cropland]
breed [farms farm]


farms-own [
  winery
  able-to-harvest       ;(BOOLEAN).
  Max-harvest
  total-harvested       ;(FLOAT) hl of grapes
  t-1-harvest
  harvest-if-not-flooded
  harvest-damaged
  harvest-damaged-bcplants
  harvest-lost-bcwinery
  potential-harvest-damaged-bcplants
  structural-cost
  prop-structural-cost
  production-cost            ;(FLOAT)
  vinification-cost
  reposition-cost
  t-1-vinification-cost
  reparation-cost-from-cooperative
  f-revenue
  cumulative-balance
  balance                ;(FLOAT)
  soil-reparation
  plant-reparation
  reparations
  direct-mg-cost       ; marginal cost respect variations of self production
  crossed-mg-cost     ; marginal cost respect variations of other's production
  croplands-owned
  croplands-in-prone-area
  croplands-impacted
  croplands-unproductive-due-destruction
  croplands-unproductive
  vinif-cost-change-self
  vinif-cost-change-others
  outsourcing-cost
  insurance
]



cooperatives-own [
  able-to-produce       ;(BOOLEAN).
  able-to-collect       ;(BOOLEAN).
  total-collected
  collected-if-not-flood
  production            ;(FLOAT) hl of wine
  production-if-not-flood
  t-1-production
  technology            ;(FLOAT) Efficiency parameter
  fixed-vinif-cost-prop            ;(FLOAT)
  t-1TC
  vinif-cost-hl
  fixed-vinif-cost
  vble-vinif-cost-hl
  vble-vinif-cost         ;(FLOAT)
  costs
  reparations
  c-revenue                ;(FLOAT)
  balance                ;(FLOAT)
  farmers-associated
  MaxVol

]



turtles-own [
  damage                ;(FLOAT) Monetary assessment of the damage; needed for reparations.
  a-flooded             ;(BOOLEAN).
   distance-to-river
   couleur
   flood-memory
 ]



croplands-own [
  owned-by              ;farm that owns the land ;(INTEGER) It takes values from a predefined vector of values = number assigned to the farm
  productive               ;(BOOLEAN).yes=1 no=0
  age                   ;(INTEGER) < age-old
  productivity          ;distance to the river, that implies different fertility. Hectolitres by hectarea
  extent             ;(FLOAT) ha of land
  yield                 ;(FLOAT) Production of the field. Hectolitres by hectarea
  quality               ;(INTEGER) It takes values from a predefined vector of qualities.
  prop-harvest-damaged       ;(FLOAT) % to apply to yield.
  soil                  ;(BOOLEAN) soil damaged 0; not damaged 1
  plants       ;(BOOLEAN) If plants destroyed/damaged 0; not destroyed/damaged 1
  plant-damage
  plant-damage_CA
  totalIdeal
  destruction-memory
  unproductive-due-destruction  ;(BOOLEAN).yes=1 no=0
  effective-cost
  operational-cost

]


patches-own [
   p-productivity
]



;;;;;SET UP;;;;;

to setup
  clear-all
  ;random-seed 0
  setup-globals

  ask patches [set pcolor 36 set p-productivity productivity-ha]

  setup-cooperatives
  setup-farms
  setup-croplands

  adjust-globals
  reset-ticks
end


;;;SET-UP SUBROUTINES


to setup-cooperatives
   let file "input/cooperatives.txt"
  ;; We check to make sure the file exists first
  ifelse ( file-exists? file )[
    set input-data []
    ;; This opens the file, so we can use it.
    file-open file
    while [ not file-at-end? ][
      ;; Read in all the data in the file
      set input-data sentence input-data (list ( list file-read file-read file-read file-read file-read file-read))
    ]

    ;; Done reading in patch information.  Close the file.
    file-close
    ]
    [user-message (word file " No such file!") ]

   foreach input-data [create-cooperatives 1 [;set who item 0 ?
                                            set xcor item 1 ?
                                            set distance-to-river item 1 ?
                                            set ycor item 2 ?
                                            set technology item 3 ?
                                            set fixed-vinif-cost-prop item 4 ?
                                            set vinif-cost-hl item 5 ?
                                            set shape "circle"
                                            set size 6
                                            set color 0
                                            set able-to-collect 1
                                            set able-to-produce 1
                                            set technology 1
                                            set flood-memory (list)
                                            set reparations 0
                                            ]]
end


to setup-farms
  let file "input/farms.txt"
  ;; We check to make sure the file exists first
  ifelse ( file-exists? file )[
    set input-data []
    ;; This opens the file, so we can use it.
    file-open file
    while [ not file-at-end? ][
      ;; Read in all the data in the file
      set input-data sentence input-data (list ( list file-read file-read file-read file-read file-read file-read))
    ]

    ;; Done reading in patch information.  Close the file.
    file-close
    ]
    [user-message (word file " No such file!") ]

  foreach input-data [ create-farms 1 [ set xcor item 1 ?
                                        set distance-to-river item 1 ?
                                        set ycor item 2 ?
                                        set cumulative-balance item 3 ?
                                        set prop-structural-cost item 4 ?
                                        set winery item 5 ?
                                        set shape "circle"
                                        set size 3
                                        set able-to-harvest 1
                                        set flood-memory (list)
                                        set reparations 0
                                        ]]

    ask cooperatives [
        set farmers-associated (sentence [who] of farms with [winery = myself])
        set farmers-associated sort farmers-associated
    ]
end





to setup-croplands
  let file (word "input/croplands" dist-control ".txt")
  ;; We check to make sure the file exists first
  ifelse ( file-exists? file )[
    set input-data []
    ;; This opens the file, so we can use it.
    file-open file
    while [ not file-at-end? ][
      ;; Read in all the data in the file
      set input-data sentence input-data (list ( list file-read file-read file-read file-read file-read file-read file-read file-read file-read))
    ]

    ;; Done reading in patch information.  Close the file.
    file-close
    ]
    [user-message (word file " No such file!") ]
  foreach input-data [create-croplands 1 [set color item 1 ?
                                          set distance-to-river item 2 ?
                                          set xcor item 2 ?
                                          set ycor item 3 ?
                                          set productive item 4 ?
                                          set age item 5 ?
                                          set extent item 6 ?
                                          set operational-cost item 7 ?
                                          set owned-by item 8 ?
                                          set shape "circle"
                                          set size 1
                                          set productivity [p-productivity] of patch-here
                                          set flood-memory (list)
                                          set destruction-memory (list)
                                          set plants 1
                                          set plant-damage 0
                                          set plant-damage_CA 0
                                          set prop-harvest-damaged 0
                                          set soil 1
                                          set unproductive-due-destruction 0
                                          ]]
  end

to setup-globals
  ;set seasonal-operational-cost [0.26 0.32 0.25 0.17] ;proposal following Pauline's
  ;set seasonal-operational-cost [0.3 0.4 0.1 0.2] ;first proposal
  set seasonal-operational-cost [0.45 0.16 0.29 0.1] ; proposal following pauline's but assigning tasks to seasons instead of allowing the to be extended along different seasons
  set season [ "winter" "spring" "summer" "autumn"]
  ;set season ["spring" "summer" "autumn" "winter"]
  set results []



  let file "input/globals.txt"

  ;; We check to make sure the file exists first
  ifelse ( file-exists? file )[
    ;; This opens the file, so we can use it.
    file-open file
    while [ not file-at-end? ][
      ;; Read in all the data in the file
      set age-productive file-read
      set age-old file-read
      set nbFarmPerCoop file-read
      set nbCoop file-read
      set nbLandPerFarm file-read
      set soil-reparation-cost file-read
      set plant-replacement-cost file-read
      set outsource-increment file-read
      set farm-reparation-cost file-read
          if CatNat-CA? = "yes" and (0.1 * farm-reparation-cost) >= 1140 [set farm-insured-cost (0.9 * farm-reparation-cost)
                                                                          set farm-reparation-cost (0.1 * farm-reparation-cost)]
      set coop-reparation-cost file-read
          if CatNat-CA? = "yes" and (0.1 * coop-reparation-cost) >= (1140 / nbFarmPerCoop) [set coop-insured-cost (0.9 * coop-reparation-cost)
                                                                                            set coop-reparation-cost (0.1 * coop-reparation-cost)]
    set productivity-ha file-read
    set wine-prices file-read
    ]

    ;; Done reading in patch information.  Close the file.
    file-close
    ]
    [user-message (word file " No such file!") ]


  if TacticalBehavior = "outsourcing" [
    set ConseqCost [1.8 1.8 1.8 1.8]
    set ConseqHarv [0 0 0 0]
    set outsource-increment [0.8 0.8 0.8 0.8]
    ]
  if TacticalBehavior = "autonomous" [
    set ConseqCost [0.5 0.5 0.5 0.5]
    set ConseqHarv [0.365 0.185 0.215 0.5]
    set outsource-increment [0 0 0 0]
    ]
end

to adjust-globals
  ask farms[
           set croplands-owned count croplands with [owned-by = myself]
           set croplands-in-prone-area (count croplands with [owned-by = myself and xcor <= Extent])/ croplands-owned
           set croplands-unproductive count croplands with [age < age-productive and owned-by = myself]
           set structural-cost (prop-structural-cost / (1 - prop-structural-cost)) * croplands-owned * [operational-cost] of one-of croplands with [owned-by = myself]
           set Max-harvest sum [productivity * extent] of croplands with [owned-by = myself]
           set t-1-harvest sum [productivity * extent] of croplands with [owned-by = myself and age >= age-productive]
           set total-harvested t-1-harvest
            ]
  ask cooperatives [
           set vble-vinif-cost-hl ((1 - fixed-vinif-cost-prop) * vinif-cost-hl)
           set MaxVol sum [Max-harvest] of farms with [winery = myself]
           set fixed-vinif-cost (fixed-vinif-cost-prop * vinif-cost-hl * MaxVol)
           set t-1-production technology * sum [t-1-harvest] of farms with [winery = myself]
           set t-1TC fixed-vinif-cost + reparations + (vble-vinif-cost-hl * t-1-production)
           set total-collected sum [t-1-harvest] of farms with [winery = myself]
           ]
  foreach sort cooperatives [ask farms with [winery = ?][ set t-1-vinification-cost  (t-1-harvest * [t-1TC / t-1-production] of ?)
                                                          set crossed-mg-cost (-1 * t-1-harvest * [(fixed-vinif-cost + reparations) / (t-1-production ^ 2)] of ?)
                                                          set direct-mg-cost ([vble-vinif-cost-hl + (fixed-vinif-cost + reparations) / t-1-production] of ?) + crossed-mg-cost]]
  ;ask farms [create-links-with croplands with [owned-by = myself]]
end



;;;;;GO PROCEDURES;;;;;

to go
  ;random-seed 0
  clear-flood
  season-dynamics
  if current-season = "winter" [ask farms [set reparations 0
                                           set reparation-cost-from-cooperative 0
                                           set soil-reparation 0
                                           set plant-reparation 0
                                           set harvest-lost-bcwinery 0
                                           set insurance 0
                                           set harvest-if-not-flooded 0
                                           set outsourcing-cost 0]
                                ask cooperatives [set reparations 0]
                                ask croplands [set plant-damage 0
                                               set plant-damage_CA 0
                                               set effective-cost 0]]
  if current-season = "spring" [ask cooperatives [set able-to-produce 1
                                                  set able-to-collect 1]]
  if ticks < 4 and current-season = flood-in-season [flood]
  calculate-damages-cropland
  calculate-damages-farms
  calculate-damage-coop
  harvest
  wine-making
  sales
  costs-revenues
  age-dynamics
  replant
  tick
  if current-season = "autumn" [store-results
                                ask farms [set t-1-harvest total-harvested
                                           set t-1-vinification-cost vinification-cost]
                                ;ask farms with [able-to-harvest = 0] [set able-to-harvest 1]
                                ask cooperatives [set t-1-production production
                                                  set t-1TC costs]
                                ask croplands [set prop-harvest-damaged 0]
                                ;ask croplands with [soil = 0] [;set prop-harvest-damaged 0
                                ;                               set soil 1
                                                              ;set a-flooded 0
                                ;                               if plants = 0 [set unproductive-due-destruction 1
                                ;                                              set plants 1
                                ;                                              set age age-old
                                ;                                              set yield 0]
                                ;                               ]
                                ]
  if ticks = ticks-stop [stop]
  ;ifelse sum [cumulative-balance] of farms > 0 [stop] [if ticks = ticks-stop [stop]]

end





;;GO SUBROUTINES

to clear-flood
  ask patches [set pcolor 36]
  ask turtles with [a-flooded = 1][set a-flooded 0]

end

to flood
  ask turtles with [xcor <= Flood-Extent][set a-flooded 1]
  ask patches with [pxcor <= Flood-Extent] [set pcolor blue]
  ask turtles [ifelse a-flooded = 1 [set flood-memory lput 1 flood-memory]
                                    [set flood-memory lput 0 flood-memory]]
end

to season-dynamics
  ifelse current-season = 0 [set current-season first season][
    let index (position current-season season + 1)
    ifelse index < 4 [set current-season item index season][set current-season first season]
  ]
  if current-season = "winter" [set year year + 1]
end


to calculate-damages-cropland
  ;random-seed 0
    ask croplands with [productive = 1 and a-flooded = 1][set soil 0]
    ask croplands with [productive = 1 and a-flooded = 1][if plants = 1 and dam_byR = false [if current-season = "spring" [ifelse random 100 < 50 [set plants 0 set yield 0 set prop-harvest-damaged 1]
                                                                                                                                                  [set prop-harvest-damaged 0.5 ]
                                                                                                                           ]
                                                                                             if current-season = "summer" [ifelse random 100 < 20 [set plants 0 set yield 0 set prop-harvest-damaged 1]
                                                                                                                                                  [set prop-harvest-damaged 1]
                                                                                                                           ]
                                                                                             if current-season = "autumn" [ifelse random 100 < 10 [set plants 0 set yield 0 set prop-harvest-damaged 1]
                                                                                                                                                  [set prop-harvest-damaged 1]
                                                                                                                           ]
                                                                                             ]
                                                          if plants = 1 and dam_byR = true  [if current-season = "spring" [set prop-harvest-damaged 0.5]
                                                                                             if current-season = "summer" [set prop-harvest-damaged 1]
                                                                                             if current-season = "autumn" [set prop-harvest-damaged 1]
                                                                                             if current-season != "winter" [foreach cropland-dest [ask cropland ? [set plants 0 set yield 0 set prop-harvest-damaged 1]]]
                                                                                             ]
                                                               ]
    ask croplands [ifelse plants = 0 [set destruction-memory lput 1 destruction-memory
                                      set unproductive-due-destruction 1
                                      if age >= age-productive [set plant-damage_CA (1313 * age-productive) + (plant-replacement-cost - ((plant-replacement-cost / age-old) * (age - age-productive)))]
                                      if age < age-productive [set plant-damage_CA (1313 * age-productive) + ((plant-replacement-cost / 2) + ((age - 1)* 1542))]
                                      ;1313 = gross benefit per ha. Calculated outside the model with initialization values given in ODD
                                      ;1542 = Cost per ha when plot unproductive. Calculated outside the model with initialization values given in ODD
                                      set plant-damage plant-replacement-cost
                                      ]
                                      [set destruction-memory lput 0 destruction-memory]]

    ask farms with [a-flooded = 1] [let aux position current-season season
                                    ask croplands with [owned-by = myself and age >= age-productive and plants = 1 and prop-harvest-damaged < 1][
                                    ifelse prop-harvest-damaged = 0 [set prop-harvest-damaged item aux ConseqHarv]
                                                               [set prop-harvest-damaged prop-harvest-damaged + (prop-harvest-damaged * item aux ConseqHarv)]
                                      ]
                                    ]


end

;ifelse (age-old - age) >= 5 [set plant-damage (wine-prices * productivity * extent ) * 5 + (plant-replacement-cost * (age-old - age) / (age-old - age-productive))]
;                                                                  [set plant-damage ((wine-prices * productivity * extent ) + (plant-replacement-cost / (age-old - age-productive)))* (age-old - age)]

to calculate-damages-farms
  ask farms [set croplands-impacted count croplands with [a-flooded = 1 and owned-by = myself]]
  if current-season = "autumn" [ask farms [set harvest-damaged sum [productivity * extent * prop-harvest-damaged] of croplands with [owned-by = myself and age >= age-productive]
                                           set harvest-damaged-bcplants sum [productivity * extent] of croplands with [owned-by = myself and age >= age-productive and unproductive-due-destruction = 1]
                                           set potential-harvest-damaged-bcplants sum [productivity * extent] of croplands with [owned-by = myself and unproductive-due-destruction = 1]
                                          ]
                                ask farms with [a-flooded = 1][set able-to-harvest 0]]

  if current-season != "autumn" [ask farms with [a-flooded = 1] [set reparations reparations + farm-reparation-cost
                                                                 if CatNat-CA? = "yes" [set insurance insurance + farm-insured-cost]
                                                                     ]]

  if current-season = "autumn" [ask farms [let aux count croplands with [soil = 0 and owned-by = myself]
                                           let aux2 count croplands with [plants = 0 and owned-by = myself]
                                           set soil-reparation (aux * soil-reparation-cost)
                                           set plant-reparation sum [plant-damage] of croplands with [owned-by = myself]
                                           if CatNat-CA? = "yes" [
                                              if (harvest-damaged * wine-prices) + soil-reparation + plant-reparation >= (0.3 * Max-harvest * wine-prices) [set insurance insurance + (0.35 * soil-reparation)
                                                                                                                                                            set insurance insurance + (0.25 * sum [plant-damage_CA] of croplands with [owned-by = myself])
                                                                                                                                                            set insurance insurance + (0.25 * harvest-damaged * wine-prices)
                                                                                                                                                            set soil-reparation (0.65 * soil-reparation)
                                                                                                                                                            set plant-reparation plant-reparation - (0.25 * sum [plant-damage_CA] of croplands with [owned-by = myself])
                                                                                                                                                            set harvest-damaged (0.75 * harvest-damaged)]]
                                           set reparations reparations + soil-reparation + plant-reparation
                                          ]
                                ;;; farms' buildings reparation
                                ask farms with [a-flooded = 1][set reparations reparations + farm-reparation-cost
                                                               if CatNat-CA? = "yes" [set insurance insurance + farm-insured-cost]
                                                              ]
                                ask farms with [able-to-harvest = 0] [set able-to-harvest 1
                                                                     ;set a-flooded 0
                                                                     ]

                                ask croplands with [soil = 0] [;set prop-harvest-damaged 0
                                                               set soil 1
                                                              ;set a-flooded 0
                                                               if plants = 0 [set unproductive-due-destruction 1
                                                                              set plants 1
                                                                              set age age-old
                                                                              set yield 0]
                                                               ]
                               ]
end


to calculate-damage-coop
  ask cooperatives with [a-flooded = 1][set reparations reparations + coop-reparation-cost
                                        let aux reparations / t-1-production
                                        ask farms with [winery = myself][set reparation-cost-from-cooperative (aux * t-1-harvest)]
                                        if current-season = "autumn" [set able-to-collect 0]
                                        ]
  if current-season = "winter" [ask cooperatives with [a-flooded = 1 or able-to-collect = 0][set able-to-produce 0]]

end

to harvest
  if current-season = "autumn" [
    ask croplands with [plants = 1 and age >= age-productive]  [set yield (productivity * extent * (1 - prop-harvest-damaged))]
    ask farms [set total-harvested sum [yield] of croplands with [owned-by = myself]]
    ;ask farms with [able-to-harvest = 0] [ask croplands with [owned-by = myself and age >= age-productive and plants = 1 and prop-harvest-damaged < 1][set effective-cost effective-cost + (outsource-increment * operational-cost)]
    ;                                      set outsourcing-cost sum [outsource-increment * operational-cost] of croplands with [owned-by = myself and age >= age-productive and plants = 1 and prop-harvest-damaged < 1]
                                          ;]
    ;ask croplands [set prop-harvest-damaged 0]
  ]
end

to wine-making
  if current-season = "autumn" [
    ask cooperatives [
      set total-collected sum [total-harvested] of farms with [winery = myself]
    ]
    ask cooperatives with [able-to-collect = 0] [set collected-if-not-flood total-collected
                                                    set total-collected  0
                                                    ask farms with [winery = myself][set harvest-lost-bcwinery total-harvested
                                                                                     set total-harvested 0]
                                                    ]

  ]
  if current-season = "winter" [
    ask cooperatives [
      set production total-collected * technology
    ]
    ask cooperatives with [able-to-produce = 0] [set production-if-not-flood production
                                                    set production  0
                                                    ask farms with [winery = myself][set harvest-lost-bcwinery (harvest-lost-bcwinery + total-harvested)
                                                                                     set total-harvested 0]
                                                    ]
  ]
end
to sales
if current-season = "spring"[ask cooperatives [set c-revenue wine-prices * production]
                             ask cooperatives with[a-flooded = 1][set c-revenue 0
                                                                  ask farms with [winery = myself][set harvest-lost-bcwinery total-harvested]
                                                    ]]
end

;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;agents-choices
;;;;;;;;;;;;;;;;;;;;;;;;


to replant

  if current-season = "winter" [
  if replant? = 1 [ask croplands with [productive = 0] [set productive 1]]
  if replant? = "random" [ask croplands with [productive = 0] [set productive random 2]]
  if replant? = 0 [ask croplands with [productive = 0] [set productive 0] ]
  ask farms [let aux count croplands with [productive = 1 and age = 0 and owned-by = myself and unproductive-due-destruction = 0]
                                         set reposition-cost plant-replacement-cost * aux]
  ]
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;COST-REVENUES SHARING;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to costs-revenues
  ;;update marginal costs
  if current-season = "spring" [
      foreach sort cooperatives [
        ifelse [production] of ? = 0 [ask farms with [winery = ?][set crossed-mg-cost 0 set direct-mg-cost 0]]
                                   [ask farms with [winery = ?][
                                  set crossed-mg-cost (-1 * total-harvested * [(fixed-vinif-cost + reparations) / (production ^ 2)] of ?)
                                  set direct-mg-cost ([vble-vinif-cost-hl + (fixed-vinif-cost + reparations) / production] of ?) + crossed-mg-cost]
                                   ]]]

  ;;;Regarding cooperatives
  ask cooperatives [
    if current-season = "spring" [

      ;:cooperatives' variable cost, revenue and financial balance calculus
      set vble-vinif-cost (vble-vinif-cost-hl * total-collected)
      ;set c-revenue wine-prices * production
      set costs vble-vinif-cost + fixed-vinif-cost + reparations
      set balance (c-revenue - vble-vinif-cost - fixed-vinif-cost - reparations)

      ;;; revenues sharing with farmers
      ifelse production = 0 [ask farms with [winery = myself] [set f-revenue 0]][let aux (c-revenue / total-collected)
                                                                                 ask farms with [winery = myself] [set f-revenue aux * total-harvested]
                                                                                 ]
      ;;sharing costs with farmers
      ifelse production = 0 [let aux (fixed-vinif-cost) / length farmers-associated
                             ask farms with [winery = myself] [set vinification-cost aux]

                             ;let aux2 reparations / length farmers-associated
                             ;ask farms with [winery = myself] [set reparation-cost-from-cooperative aux2]

                             if CatNat-CA? = "yes" [
                               if reparations != 0 [let aux3 coop-insured-cost / length farmers-associated
                                                    ask farms with [winery = myself][set insurance insurance + aux3]]]
                                                    ]

                            [let aux (vble-vinif-cost + fixed-vinif-cost) / total-collected
                             ask farms with [winery = myself] [set vinification-cost (aux * total-harvested)]

                             ;;reparations
                             ;let aux2 reparations / total-collected
                             ;ask farms with [winery = myself] [set reparation-cost-from-cooperative (aux2 * total-harvested)]
                             if CatNat-CA? = "yes" [
                               if reparations != 0 [let aux3 coop-insured-cost / total-collected
                                                    ask farms with [winery = myself][set insurance insurance + (aux3 * total-harvested)]]]
                             ]

                            ]]


;;;Regarding plots

;ask croplands with [age >= age-productive and plants = 1 and prop-harvest-damaged < 1][let aux position current-season season
;                                                          set effective-cost effective-cost + (item aux seasonal-operational-cost * operational-cost)]

ask croplands with [age < age-productive and plants = 1][let aux position current-season season
                                                          set effective-cost effective-cost + (item aux seasonal-operational-cost * operational-cost * 0.2)]

;;;Regarding farms
ask farms with [a-flooded = 1] [let aux position current-season season
                                ask croplands with [owned-by = myself and age >= age-productive and plants = 1 and prop-harvest-damaged < 1][set effective-cost effective-cost + (item aux ConseqCost * item aux seasonal-operational-cost * operational-cost)]
                                set outsourcing-cost sum [item aux outsource-increment * item aux seasonal-operational-cost * operational-cost] of croplands with [owned-by = myself and age >= age-productive and plants = 1 and prop-harvest-damaged < 1]]

ask farms with [a-flooded = 0] [let aux position current-season season
                                ask croplands with [owned-by = myself and age >= age-productive and plants = 1 and prop-harvest-damaged < 1][set effective-cost effective-cost + (item aux seasonal-operational-cost * operational-cost)]
                                ]

if current-season = "autumn" [ask farms [;let aux count croplands with [productive = 1 and age = 0 and owned-by = myself and unproductive-due-destruction = 0]
                              			 ;set reposition-cost plant-replacement-cost * aux
                                     set production-cost structural-cost + sum [effective-cost] of croplands with [owned-by = myself]
                              			 set balance (f-revenue - production-cost - reposition-cost - vinification-cost - reparations - reparation-cost-from-cooperative)
                              			 set cumulative-balance (cumulative-balance + balance)
                              			 set vinif-cost-change-self direct-mg-cost * (total-harvested - t-1-harvest)]
                              foreach sort cooperatives [ask farms with [winery = ?][ set vinif-cost-change-others crossed-mg-cost * ([production - t-1-production] of ? - total-harvested + t-1-harvest)]]
                             ]



  ;ask farms with [reparations != 0][set reparations 0]
end

to age-dynamics

  if current-season = "autumn" [ask croplands with [plants = 1 and productive = 1][
    set age (age + 1)
    if age >= age-old [set age 0 set yield 0 set productive 0]
    ]
  ask croplands with [plants = 0] [set age 0 set productive 0]
  ask farms [set croplands-unproductive-due-destruction count croplands with [unproductive-due-destruction = 1 and owned-by = myself]]
  ask croplands with [unproductive-due-destruction = 1 and age >= age-productive][set unproductive-due-destruction 0]
  ask farms [set croplands-unproductive count croplands with [age < age-productive and owned-by = myself]]
  ]
end

to store-results
  ask farms [
    set results sentence results (list (list  who
                                              year
                                              a-flooded
                                              able-to-harvest
                                              total-harvested
                                              t-1-harvest
                                              harvest-if-not-flooded
                                              production-cost
                                              vinification-cost
                                              t-1-vinification-cost
                                              reparation-cost-from-cooperative
                                              outsourcing-cost
                                              insurance
                                              f-revenue
                                              cumulative-balance
                                              balance
                                              soil-reparation
                                              plant-reparation
                                              reposition-cost
                                              reparations
                                              croplands-owned
                                              croplands-in-prone-area
                                              croplands-impacted
                                              croplands-unproductive-due-destruction
                                              croplands-unproductive
                                              direct-mg-cost
                                              crossed-mg-cost
                                              vinif-cost-change-self
                                              vinif-cost-change-others
                                              harvest-damaged
                                              harvest-damaged-bcplants
                                              potential-harvest-damaged-bcplants
                                              harvest-lost-bcwinery
                                              Flood-Extent
                                              flood-in-season-nb
                                              dist-control))
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
239
19
687
488
-1
-1
2.901
1
3
1
1
1
0
0
0
1
0
150
0
150
0
0
1
ticks
30.0

BUTTON
99
15
166
49
NIL
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
768
19
863
64
Season
current-season
17
1
11

CHOOSER
20
71
212
116
flood-in-season
flood-in-season
"spring" "summer" "autumn" "winter"
2

MONITOR
707
19
764
64
Year
year
17
1
11

BUTTON
21
15
95
49
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
19
119
213
152
Flood-Extent
Flood-Extent
0
maxExtent
100
1
1
NIL
HORIZONTAL

INPUTBOX
19
158
104
218
maxExtent
100
1
0
Number

TEXTBOX
19
223
169
253
Farm-plot coupling dist:
12
0.0
1

TEXTBOX
20
55
170
73
FLOOD parameters
12
0.0
1

CHOOSER
22
325
114
370
replant?
replant?
1 "random" 0
0

PLOT
706
225
1381
413
Farms
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Gross benefit" 1.0 0 -7500403 true "" "plot sum [balance] of farms"
"Revenues" 1.0 0 -2674135 true "" "plot sum [f-revenue] of farms"
"Vinifiction and production costs" 1.0 0 -955883 true "" "plot sum [production-cost + vinification-cost] of farms"
"Cumulative gross balance" 1.0 0 -6459832 true "" "plot sum [cumulative-balance] of farms"
"Cat-Nat + Calamite agricole" 1.0 0 -5825686 true "" "plot sum [insurance] of farms"
"Production costs" 1.0 0 -10899396 true "" "plot sum [production-cost] of farms"

PLOT
707
69
1287
219
Cooperatives
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Production" 1.0 0 -16777216 true "" "plot sum [production] of cooperatives"
"Benefits" 1.0 0 -7500403 true "" "plot sum [balance] of cooperatives"
"Revenues" 1.0 0 -2674135 true "" "plot sum [c-revenue] of cooperatives"
"Vinification cost" 1.0 0 -955883 true "" "plot sum [costs] of cooperatives"

MONITOR
869
19
1044
64
Improductive croplands (%)
(count croplands with [age < age-productive]/ count croplands) * 100
2
1
11

MONITOR
1216
19
1291
64
Croplands
count croplands
2
1
11

MONITOR
1046
19
1212
64
Croplands prone area (%)
(count croplands with [xcor < 100]/ count croplands) * 100
2
1
11

PLOT
706
415
1369
578
Effects of flood
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Yield lost" 1.0 0 -16777216 true "" "plot (sum [harvest-if-not-flooded + harvest-damaged] of farms) * wine-prices"
"Plants, soils, buildings" 1.0 0 -7500403 true "" "plot sum [reparations + reparation-cost-from-cooperative] of farms"
"Cat-Nat + Calamite Agricole" 1.0 0 -955883 true "" "plot sum [insurance] of farms"
"Delayed effects over yield" 1.0 0 -2674135 true "" "plot (sum [potential-harvest-damaged-bcplants] of farms) * wine-prices"
"Yield lost indirectly" 1.0 0 -6459832 true "" "plot (sum [harvest-if-not-flooded] of farms) * wine-prices"
"Yield lost directly" 1.0 0 -1184463 true "" "plot (sum [harvest-damaged] of farms) * wine-prices"
"Outsourcing cost" 1.0 0 -10899396 true "" "plot sum [outsourcing-cost] of farms"

CHOOSER
21
373
212
418
CatNat-CA?
CatNat-CA?
"no" "yes"
1

INPUTBOX
110
158
213
218
flood-in-season-nb
3
1
0
Number

TEXTBOX
21
305
171
323
Other parameters:
12
0.0
1

INPUTBOX
22
425
75
485
ticks-stop
120
1
0
Number

INPUTBOX
24
241
90
301
dist-control
1
1
0
Number

TEXTBOX
28
501
178
519
Tactical behavior
12
0.0
1

CHOOSER
22
519
164
564
TacticalBehavior
TacticalBehavior
"outsourcing" "autonomous"
0

SWITCH
83
426
203
459
dam_byR
dam_byR
1
1
-1000

@#$#@#$#@
## WHAT IS IT?

Cooperative systems of wine production. Based on Southern France area.
TOY Model!!! David Nortes Martinez (IRSTEA). April 7th, 2016

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

farm
false
2
Rectangle -1 true false 90 150 150 180
Polygon -955883 true true 90 150 120 120 150 150
Polygon -1 true false 150 180 180 150 180 120 150 150 150 180
Line -16777216 false 150 150 150 180
Polygon -955883 true true 120 120 150 105 180 120 150 150 120 120
Line -6459832 false 120 120 150 150
Polygon -6459832 true false 180 150 240 150 210 180 150 180
Line -10899396 false 180 180 210 150
Line -10899396 false 165 180 195 150
Line -10899396 false 195 180 225 150
Rectangle -16777216 true false 105 165 135 180

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

industry
false
3
Rectangle -13791810 true false 105 135 180 180
Polygon -13345367 true false 180 180 225 135 226 96 180 135 180 180
Line -16777216 false 180 135 180 180
Polygon -7500403 true false 105 135 165 96 225 96 180 135 105 135
Line -16777216 false 180 136 224 96
Line -16777216 false 105 135 180 135
Line -16777216 false 165 96 165 105
Line -16777216 false 105 180 180 180
Line -16777216 false 225 135 180 180
Polygon -6459832 true true 165 105 214 106 180 135 120 135 165 105
Rectangle -13345367 true false 45 75 45 90
Rectangle -13345367 true false 60 90 75 90
Line -7500403 false 120 135 120 180
Line -7500403 false 165 135 165 180
Line -7500403 false 135 135 135 180
Line -7500403 false 105 150 180 150
Line -7500403 false 105 165 180 165
Line -7500403 false 196 122 195 165
Line -7500403 false 180 150 225 105
Line -7500403 false 180 165 225 120
Line -16777216 false 165 105 120 135
Line -16777216 false 165 105 215 105
Line -7500403 false 210 110 210 150
Line -7500403 false 150 135 150 180

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

winegrapes
false
0
Circle -8630108 true false 135 120 30
Circle -8630108 true false 120 150 30
Circle -8630108 true false 135 180 30
Polygon -13840069 true false 150 120 120 90 150 105 150 120 180 90 150 105
Circle -5825686 true false 105 120 30
Circle -5825686 true false 150 150 30
Circle -5825686 true false 165 120 30

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.3.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="cashflow" repetitions="5000" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <enumeratedValueSet variable="nbCoop">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="maxExtent">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Flood-Extent">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="seed">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="flood-in-season">
      <value value="&quot;winter&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Load-data-from-file">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="outsourcing?">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Fixed-vinif-cost-proportion">
      <value value="0.2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="replant?">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nbLandPerFarm">
      <value value="26"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="scenario-floods">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nbFarmPerCoop">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Insurance?">
      <value value="&quot;yes&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Create-output-file">
      <value value="true"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="Article0" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>count turtles</metric>
    <enumeratedValueSet variable="flood-in-season">
      <value value="&quot;spring&quot;"/>
      <value value="&quot;autumn&quot;"/>
      <value value="&quot;summer&quot;"/>
      <value value="&quot;winter&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="maxExtent">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Insurance?">
      <value value="&quot;yes&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="outsourcing?">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="replant?">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="scenario-floods">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="seed">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Create-output-file">
      <value value="true"/>
    </enumeratedValueSet>
    <steppedValueSet variable="Flood-Extent" first="1" step="1" last="99"/>
    <steppedValueSet variable="dist-control" first="1" step="1" last="5"/>
  </experiment>
  <experiment name="experiment" repetitions="1" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>sort [who] of croplands with [unproductive-due-destruction = 1]</metric>
    <metric>sort [who] of croplands with [a-flooded = 1]</metric>
    <enumeratedValueSet variable="dist-control">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="flood-in-season">
      <value value="&quot;spring&quot;"/>
      <value value="&quot;summer&quot;"/>
      <value value="&quot;autumn&quot;"/>
      <value value="&quot;winter&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="maxExtent">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="TacticalBehavior">
      <value value="&quot;outsourcing&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="replant?">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ticks-stop">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="flood-in-season-nb">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="CatNat-CA?">
      <value value="&quot;yes&quot;"/>
    </enumeratedValueSet>
    <steppedValueSet variable="Flood-Extent" first="0" step="1" last="99"/>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
