#~~~~ reading packages
library(RNetLogo)
library(svDialogs)

#~~~~ Path declaration
main.path = getwd() 
jar.path = dlgDir(default = getwd(), title = "Please introduce your Netlogo.jar folder path",gui = .GUI)$res
m3.path = file.path(main.path, "EMS-Model3.nlogo")
output.path = file.path(main.path, "output")
input.path = file.path(main.path, "input")
config = 1  #Topology to simulate. It admits numbers in c=(1, 6,7,8,9,10,11,12,13,14)
#~~~~ Sourcing functions
source(file.path(main.path, "EMS-Functions.R"))


#~~~~ Declaring output's names of variables
vblesnames <- c('agent',
                'year',
                'a.flooded',
                'able.to.harvest',
                'total.harvested',
                't.1.harvest',
                'harvest.if.not.flooded',
                'production.cost',
                'vinification.cost',
                't.1.vinification.cost',
                'reparation.cost.from.cooperative',
                'outsourcing.cost',
                'insurance',
                'revenue',
                'cumulative.balance', 
                'balance',
                'soil.reparation',
                'plant.reparation',
                'plant.repositions',
                'reparations',
                'croplands.owned',
                'croplands.in.prone.area',
                'croplands.impacted',
                'croplands.unproductive.due.destruction',
                'croplands.unproductive',
                'direct.mg.cost',
                'crossed.mg.cost',
                'vinif.cost.change.self',
                'vinif.cost.change.others',
                'harvest.damaged',
                'harvest.damaged.bcplants',
                'potential.harvest.damaged.bcplants',
                'harvest.lost.bcwinery',
                'flood',
                'fseason',
                'dist.ct')

#~~~~ setting up experiment
EP <- readRDS(file = file.path(input.path,"experiment_plan.rds"))
EP <- EP[(!EP$flood.ext %in% seq(15,100,5)) & EP$config %in% config, ]

#~~~~ initialization of the simulation: launching netlogo
nl.id = unique(EP$ID)                                        #~~~~ ID of the netlogo instance
NLStart(nl.path = jar.path, nl.obj = nl.id,  gui = F)         #~~~~ Starting netlogo
NLLoadModel(m3.path, nl.obj = nl.id)                          #~~~~ Starting netlogo's model
#~~~~ launching simulation on the opened instance
for(i in 1:nrow(EP)){
  results.db <- launchsim_winpos(nl.id = EP$ID[i],
                                 ticks.stop = EP$ticks.stop[i],
                                 tactic = EP$tactic[i],
                                 config = EP$config[i],
                                 season = EP$season[i],
                                 floodextent = EP$flood.ext[i],
                                 dam_byR = EP$dam_byR[i],
                                 winpos = EP$winpos[i],                   
                                 plotdam = ch_to_nl.list(EP$damplot[i]))
  #~~~~ database treatment
  results.db <- matrix(unlist(results.db), ncol=36, byrow=TRUE)
  colnames(results.db) <- vblesnames
  results.db <- data.frame(results.db, row.names = NULL)
  #~~~~ storing database
  saveRDS(results.db,
          file = file.path(output.path, 
                           paste("m3_",
                                 EP$season[i], "_",
                                 EP$flood.ext[i], "_conf",
                                 EP$config[i], "_",
                                 EP$tactic[i], "_",
                                 EP$winpos[i], "_rep",
                                 EP$rep[i], ".rds",
                                 sep = "")), 
          compress = TRUE)

}

#~~~~ Closing Netlogo
NLQuit(nl.obj = nl.id)

